@@ -1,51 +1,44 @@
 #/*##########################################################################
-# Copyright (C) 2004-2010 European Synchrotron Radiation Facility
+# Copyright (C) 2004-2012 European Synchrotron Radiation Facility
 #
-# This file is part of the PyMCA X-ray Fluorescence Toolkit developed at
-# the ESRF by the Beamline Instrumentation Software Support (BLISS) group.
+# This file is part of the PyMca X-ray Fluorescence Toolkit developed at
+# the ESRF by the Software group.
 #
-# This toolkit is free software; you can redistribute it and/or modify it 
+# This toolkit is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by the Free
-# Software Foundation; either version 2 of the License, or (at your option) 
+# Software Foundation; either version 2 of the License, or (at your option)
 # any later version.
 #
-# PyMCA is distributed in the hope that it will be useful, but WITHOUT ANY
+# PyMca is distributed in the hope that it will be useful, but WITHOUT ANY
 # WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 # FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 # details.
 #
 # You should have received a copy of the GNU General Public License along with
-# PyMCA; if not, write to the Free Software Foundation, Inc., 59 Temple Place,
-# Suite 330, Boston, MA 02111-1307, USA.
+# PyMca; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #
-# PyMCA follows the dual licensing model of Trolltech's Qt and Riverbank's PyQt
-# and cannot be used as a free plugin for a non-free program. 
+# PyMca follows the dual licensing model of Riverbank's PyQt and cannot be
+# used as a free plugin for a non-free program.
 #
-# Please contact the ESRF industrial unit (industry@esrf.fr) if this license 
+# Please contact the ESRF industrial unit (industry@esrf.fr) if this license
 # is a problem for you.
 #############################################################################*/
-import DataObject
-import EdfFile
-import EdfFileDataSource
-import numpy.oldnumeric as Numeric
+from PyMca import DataObject
+from PyMca import EdfFile
+from PyMca import EdfFileDataSource
 import numpy
 import sys
 import os
+
+# Offer automatic conversion to HDF5 in case of lacking
+# memory to hold the Stack.
 HDF5 = False
 try:
     import h5py
-    import PyMca.phynx as phynx
     HDF5 = True
-except ImportError:
-    try:
-        import phynx
-        HDF5 = True
-    except ImportError:
-        try:
-            from xpaxs.io import phynx
-            HDF5 = True
-        except ImportError:
-            pass
+except:
+    pass
         
 
 SOURCE_TYPE = "EdfFileStack"
@@ -111,17 +104,19 @@ class EDFStack(DataObject.DataObject):
         self.onBegin(self.nbFiles)
         singleImageShape = arrRet.shape
         if (fileindex == 2) or (self.__imageStack):
+            self.__imageStack = True
             if len(singleImageShape) == 1:
                 #single line
                 #be ready for specfile stack?
-                raise IOError, "Not implemented yet"
+                self.onEnd()
+                raise IOError("Not implemented yet")
                 self.data = numpy.zeros((arrRet.shape[0],
                                            nImages,
                                            self.nbFiles),
                                            self.__dtype)
                 self.incrProgressBar=0
                 for tempEdfFileName in filelist:
-                    tempEdf=EdfFile.EdfFile(tempEdfFileName)
+                    tempEdf=EdfFile.EdfFile(tempEdfFileName, 'rb')
                     for i in range(nImages):
                         pieceOfStack=tempEdf.GetData(i)
                         self.data[:,i, self.incrProgressBar] = pieceOfStack[:]
@@ -134,14 +129,15 @@ class EDFStack(DataObject.DataObject):
                     #should I try to convert it to a standard one
                     #using a 3D matrix or keep as 4D matrix?
                     if self.nbFiles > 1:
-                        raise IOError, "Multiple files with multiple images implemented yet"
+                        raise IOError(\
+                            "Multiple files with multiple images implemented yet")
                     self.data = numpy.zeros((arrRet.shape[0],
                                                arrRet.shape[1],
                                                nImages * self.nbFiles),
                                                self.__dtype)
                     self.incrProgressBar=0
                     for tempEdfFileName in filelist:
-                        tempEdf=EdfFile.EdfFile(tempEdfFileName)
+                        tempEdf=EdfFile.EdfFile(tempEdfFileName, 'rb')
                         for i in range(nImages):
                             pieceOfStack=tempEdf.GetData(i)
                             self.data[:,:,
@@ -157,7 +153,7 @@ class EDFStack(DataObject.DataObject):
                                                  self.__dtype)
                         self.incrProgressBar=0
                         for tempEdfFileName in filelist:
-                            tempEdf=EdfFile.EdfFile(tempEdfFileName)
+                            tempEdf=EdfFile.EdfFile(tempEdfFileName, 'rb')
                             pieceOfStack=tempEdf.GetData(0)    
                             self.data[:,:, self.incrProgressBar] = pieceOfStack
                             self.incrProgressBar += 1
@@ -177,31 +173,32 @@ class EDFStack(DataObject.DataObject):
                                             os.path.dirname(filelist[0]),
                                             "HDF5 files *.h5")
                                 if not len(hdf5file):
-                                    raise IOError, "Invalid output file"
-                                hdf5file = str(hdf5file)
+                                    raise IOError("Invalid output file")
+                                hdf5file = qt.safe_str(hdf5file)
                                 if not hdf5file.endswith(".h5"):
                                     hdf5file += ".h5"
                                 hdf, self.data =  ArraySave.getHDF5FileInstanceAndBuffer(hdf5file,
-                                              (arrRet.shape[0],
-                                               arrRet.shape[1],
-                                               self.nbFiles))
+                                              (self.nbFiles,
+                                               arrRet.shape[0],
+                                               arrRet.shape[1]))
                                 self.incrProgressBar=0
                                 for tempEdfFileName in filelist:
-                                    tempEdf=EdfFile.EdfFile(tempEdfFileName)
+                                    tempEdf=EdfFile.EdfFile(tempEdfFileName, 'rb')
                                     pieceOfStack=tempEdf.GetData(0)
-                                    self.data[:,:, self.incrProgressBar] = pieceOfStack[:,:]
+                                    self.data[self.incrProgressBar,:,:] = pieceOfStack[:,:]
+                                    hdf.flush()
                                     self.incrProgressBar += 1
                                     self.onProgress(self.incrProgressBar)
                                 hdf5done = True
                         if not hdf5done:
                             for i in range(3):
-                                print "\7"
+                                print("\7")
                             samplingStep = None
                             i = 2
                             while samplingStep is None:
-                                print "**************************************************"
-                                print " Memory error!, attempting %dx%d sampling reduction " % (i,i)
-                                print "**************************************************"
+                                print("**************************************************")
+                                print(" Memory error!, attempting %dx%d sampling reduction ") % (i,i)
+                                print("**************************************************")
                                 s1, s2 = arrRet[::i, ::i].shape
                                 try:
                                     self.data = numpy.zeros((s1, s2,
@@ -212,25 +209,26 @@ class EDFStack(DataObject.DataObject):
                                     i += 1
                             self.incrProgressBar=0
                             for tempEdfFileName in filelist:
-                                tempEdf=EdfFile.EdfFile(tempEdfFileName)
+                                tempEdf=EdfFile.EdfFile(tempEdfFileName, 'rb')
                                 pieceOfStack=tempEdf.GetData(0)
                                 self.data[:,:, self.incrProgressBar] = pieceOfStack[
                                                             ::samplingStep,::samplingStep]
                                 self.incrProgressBar += 1
-                            self.onProgress(self.incrProgressBar)
-                    self.onEnd()
+                                self.onProgress(self.incrProgressBar)
+                self.onEnd()
         else:
+            self.__imageStack = False
             if len(singleImageShape) == 1:
                 #single line
                 #be ready for specfile stack?
-                raise IOError, "Not implemented yet"
+                raise IOError("Not implemented yet")
                 self.data = numpy.zeros((self.nbFiles,
                                            arrRet.shape[0],
                                            nImages),
                                            self.__dtype)
                 self.incrProgressBar=0
                 for tempEdfFileName in filelist:
-                    tempEdf=EdfFile.EdfFile(tempEdfFileName)
+                    tempEdf=EdfFile.EdfFile(tempEdfFileName, 'rb')
                     for i in range(nImages):
                         pieceOfStack=tempEdf.GetData(i)
                         self.data[self.incrProgressBar, :,i] = pieceOfStack[:]
@@ -245,7 +243,8 @@ class EDFStack(DataObject.DataObject):
                     if self.nbFiles > 1:
                         if (arrRet.shape[0] > 1) and\
                            (arrRet.shape[1] > 1):
-                                raise IOError, "Multiple files with multiple images not implemented yet"
+                                raise IOError(\
+                                    "Multiple files with multiple images not implemented yet")
                         elif arrRet.shape[0] == 1:
                             self.data = numpy.zeros((self.nbFiles,
                                                arrRet.shape[0] * nImages,
@@ -253,13 +252,14 @@ class EDFStack(DataObject.DataObject):
                                                self.__dtype)
                             self.incrProgressBar=0
                             for tempEdfFileName in filelist:
-                                tempEdf=EdfFile.EdfFile(tempEdfFileName)
+                                tempEdf=EdfFile.EdfFile(tempEdfFileName, 'rb')
                                 for i in range(nImages):
                                     pieceOfStack=tempEdf.GetData(i)
                                     self.data[self.incrProgressBar,
                                               i,:] = \
                                                               pieceOfStack[:,:]
                                 self.incrProgressBar += 1
+                                self.onProgress(self.incrProgressBar)
                         elif arrRet.shape[1] == 1:
                             self.data = numpy.zeros((self.nbFiles,
                                                arrRet.shape[1] * nImages,
@@ -267,13 +267,14 @@ class EDFStack(DataObject.DataObject):
                                                self.__dtype)
                             self.incrProgressBar=0
                             for tempEdfFileName in filelist:
-                                tempEdf=EdfFile.EdfFile(tempEdfFileName)
+                                tempEdf=EdfFile.EdfFile(tempEdfFileName, 'rb')
                                 for i in range(nImages):
                                     pieceOfStack=tempEdf.GetData(i)
                                     self.data[self.incrProgressBar,
                                               i,:] = \
                                                               pieceOfStack[:,:]
                                 self.incrProgressBar += 1
+                                self.onProgress(self.incrProgressBar)
                     else:
                         self.data = numpy.zeros((nImages * self.nbFiles,
                                                arrRet.shape[0],
@@ -281,13 +282,14 @@ class EDFStack(DataObject.DataObject):
                                                self.__dtype)
                         self.incrProgressBar=0
                         for tempEdfFileName in filelist:
-                            tempEdf=EdfFile.EdfFile(tempEdfFileName)
+                            tempEdf=EdfFile.EdfFile(tempEdfFileName, 'rb')
                             for i in range(nImages):
                                 pieceOfStack=tempEdf.GetData(i)
                                 self.data[nImages*self.incrProgressBar+i,
                                           :,:] = \
                                                           pieceOfStack[:,:]
                             self.incrProgressBar += 1
+                            self.onProgress(self.incrProgressBar)
                     self.onEnd()
                 else:
                     #this is the common case
@@ -305,8 +307,8 @@ class EDFStack(DataObject.DataObject):
                         if os.path.isdir(swapdir):
                             swapfile = '/buffer/%s1/pymcaroitool.dat' %\
                                             os.getenv('HOSTNAME')
-                        print "needed megabytes= ", needed_
-                        print "using a buffer: %s" % swapfile
+                        print("needed megabytes = %f" % needed_)
+                        print("using a buffer: %s" % swapfile)
                         if os.path.exists(swapfile):
                             os.remove(swapfile)
 
@@ -346,7 +348,7 @@ class EDFStack(DataObject.DataObject):
                                                        arrRet.shape[0],
                                                        arrRet.shape[1]),
                                                        numpy.float32)
-                                except MemoryError:
+                                except (MemoryError, ValueError):
                                     text = "Memory Error: Attempt subsampling or convert to HDF5"
                                     if HDF5 and ('PyMcaQt' in sys.modules):
                                         import PyMcaQt as qt
@@ -356,14 +358,15 @@ class EDFStack(DataObject.DataObject):
                                           "Do you want to convert your data to HDF5?\n",
                                           qt.QMessageBox.Yes,qt.QMessageBox.No)
                                         if msg == qt.QMessageBox.No:
-                                            raise MemoryError, text
+                                            raise MemoryError(text)
                                         hdf5file = qt.QFileDialog.getSaveFileName(None,
                                                     "Please select output file name",
                                                     os.path.dirname(filelist[0]),
                                                     "HDF5 files *.h5")
                                         if not len(hdf5file):
-                                            raise IOError, "Invalid output file"
-                                        hdf5file = str(hdf5file)
+                                            raise IOError(\
+                                                "Invalid output file")
+                                        hdf5file = qt.safe_str(hdf5file)
                                         if not hdf5file.endswith(".h5"):
                                             hdf5file += ".h5"
                                         hdf, self.data =  ArraySave.getHDF5FileInstanceAndBuffer(hdf5file,
@@ -371,24 +374,42 @@ class EDFStack(DataObject.DataObject):
                                                        arrRet.shape[0],
                                                        arrRet.shape[1]))               
                                     else:    
-                                        raise MemoryError, "Memory Error"
+                                        raise MemoryError(\
+                                            "Memory Error")
                     self.incrProgressBar=0
                     if fileindex == 1:
                         for tempEdfFileName in filelist:
-                            tempEdf=EdfFile.EdfFile(tempEdfFileName)
+                            tempEdf=EdfFile.EdfFile(tempEdfFileName, 'rb')
                             pieceOfStack=tempEdf.GetData(0)    
                             self.data[:,self.incrProgressBar,:] = pieceOfStack[:,:]
                             self.incrProgressBar += 1
                             self.onProgress(self.incrProgressBar)
                     else:
+                        # test for ID24 map
+                        ID24 = False
+                        if "_sample_" in filelist[0]:
+                            i0StartFile = filelist[0].replace("_sample_", "_I0start_")
+                            if os.path.exists(i0StartFile):
+                                ID24 = True
+                                id24idx = 0
+                                i0Start = EdfFile.EdfFile(i0StartFile, 'rb').GetData(0).astype(numpy.float)
+                                i0EndFile = filelist[0].replace("_sample_", "_I0end_")
+                                i0Slope = 0.0
+                                if os.path.exists(i0EndFile):
+                                    i0End = EdfFile.EdfFile(i0EndFile, 'rb').GetData(0)
+                                    i0Slope = (i0End-i0Start)/len(filelist)
                         for tempEdfFileName in filelist:
-                            tempEdf=EdfFile.EdfFile(tempEdfFileName)
-                            pieceOfStack=tempEdf.GetData(0)    
+                            tempEdf=EdfFile.EdfFile(tempEdfFileName, 'rb')
+                            if ID24:
+                                pieceOfStack=-numpy.log(tempEdf.GetData(0)/(i0Start[0,:] + id24idx * i0Slope))
+                                pieceOfStack[numpy.isfinite(pieceOfStack) == False] = 1
+                                id24idx += 1
+                            else:
+                                pieceOfStack=tempEdf.GetData(0)
                             self.data[self.incrProgressBar, :,:] = pieceOfStack[:,:]
                             self.incrProgressBar += 1
                             self.onProgress(self.incrProgressBar)
                     self.onEnd()
-
         self.__nFiles         = self.incrProgressBar
         self.__nImagesPerFile = nImages
         shape = self.data.shape
@@ -398,8 +419,12 @@ class EDFStack(DataObject.DataObject):
         if not isinstance(self.data, numpy.ndarray):
             hdf.flush()
             self.info["SourceType"] = "HDF5Stack1D"
-            self.info["McaIndex"] = 2
-            self.info["FileIndex"] = 0
+            if self.__imageStack:
+                self.info["McaIndex"] = 0
+                self.info["FileIndex"] = 1
+            else:
+                self.info["McaIndex"] = 2
+                self.info["FileIndex"] = 0
             self.info["SourceName"] = [hdf5file]
             self.info["NumberOfFiles"] = 1
             self.info["Size"]       = 1
@@ -425,12 +450,10 @@ class EDFStack(DataObject.DataObject):
         if type(filename) == type([]):
             filename = filename[0]
         if not os.path.exists(filename):
-            raise IOError,"File %s does not exists" % filename
+            raise IOError("File %s does not exists" % filename)
         name = os.path.basename(filename)
         n = len(name)
         i = 1
-        numbers = ['0', '1', '2', '3', '4', '5',
-                   '6', '7', '8','9']
         while (i <= n):
             c = name[n-i:n-i+1]
             if c in ['0', '1', '2',
@@ -460,31 +483,31 @@ class EDFStack(DataObject.DataObject):
             nchain.reverse()
             for c in nchain:
                 number += c
-            format = "%" + "0%dd" % len(number)
+            fformat = "%" + "0%dd" % len(number)
             if (len(number) + len(suffix)) == len(name):
                 prefix = ""
             else:
                 prefix = name[0:n-i+1]
             prefix = os.path.join(os.path.dirname(filename),prefix)
             if not os.path.exists(prefix + number + suffix):
-                print "Internal error in EDFStack"
-                print "file should exist:",prefix + number + suffix
+                print("Internal error in EDFStack")
+                print("file should exist: %s " % (prefix + number + suffix))
                 return
             i = 0
             if begin is None:
                 begin = 0
-                testname = prefix+format % begin+suffix
-                while not os.path.exists(prefix+format % begin+suffix):
+                testname = prefix+fformat % begin+suffix
+                while not os.path.exists(prefix+fformat % begin+suffix):
                     begin += 1
-                    testname = prefix+format % begin+suffix
+                    testname = prefix+fformat % begin+suffix
                     if len(testname) > len(filename):break
                 i = begin
             else:
                 i = begin
-            if not os.path.exists(prefix+format % i+suffix):
-                raise ValueError,"Invalid start index file = %s" % \
-                      prefix+format % i+suffix
-            f = prefix+format % i+suffix
+            if not os.path.exists(prefix+fformat % i+suffix):
+                raise ValueError("Invalid start index file = %s" % \
+                      (prefix+fformat % i+suffix))
+            f = prefix+fformat % i+suffix
             filelist = []
             while os.path.exists(f):
                 filelist.append(f)
@@ -492,7 +515,7 @@ class EDFStack(DataObject.DataObject):
                 if end is not None:
                     if i > end:
                         break
-                f = prefix+format % i+suffix
+                f = prefix+fformat % i+suffix
             self.loadFileList(filelist, fileindex=fileindex)
 
     def getSourceInfo(self):
@@ -505,7 +528,7 @@ class EDFStack(DataObject.DataObject):
         sourceInfo["KeyList"]= self.__keyList
 
     def getKeyInfo(self, key):
-        print "Not implemented"
+        print("Not implemented")
         return {}
 
     def isIndexedStack(self):
@@ -525,16 +548,16 @@ if __name__ == "__main__":
     #stack.loadIndexedStack("Z:\COTTE\ch09\ch09__mca_0005_0000_0070.edf")
     stack.loadIndexedStack(".\COTTE\ch09\ch09__mca_0005_0000_0070.edf")
     shape = stack.data.shape
-    print "elapsed = ", time.time() - t0
+    print("elapsed = %f" % (time.time() - t0))
     #guess the MCA
     imax = 0
     for i in range(len(shape)):
         if shape[i] > shape[imax]:
             imax = i
 
-    print "selections ",
-    print "getZSelectionArray  shape = ", stack.getZSelectionArray().shape
-    print "getXYSelectionArray shape = ", stack.getXYSelectionArray().shape
+    print("selections ")
+    print("getZSelectionArray  shape = ", stack.getZSelectionArray().shape)
+    print("getXYSelectionArray shape = ", stack.getXYSelectionArray().shape)
 
     import PyMcaQt as qt
     app = qt.QApplication([])
@@ -548,11 +571,11 @@ if __name__ == "__main__":
         import QtBlissGraph
         w = QtBlissGraph.QtBlissGraph()
         graph = w
-    print "shape sum 0 = ",Numeric.sum(stack.data, 0).shape
-    print "shape sum 1 = ",Numeric.sum(stack.data, 1).shape
-    print "shape sum 2 = ",Numeric.sum(stack.data, 2).shape
-    a = Numeric.sum(stack.data, imax)
-    print a.shape
+    print("shape sum 0 = ",numpy.sum(stack.data, 0).shape)
+    print("shape sum 1 = ",numpy.sum(stack.data, 1).shape)
+    print("shape sum 2 = ",numpy.sum(stack.data, 2).shape)
+    a = numpy.sum(stack.data, imax)
+    print(a.shape)
     graph.setX1AxisLimits(0, a.shape[0])
     if 0:
         w.setY1AxisLimits(0, a.shape[1])
@@ -569,9 +592,9 @@ if __name__ == "__main__":
     w.show()
 
     if imax == 0:
-        mcaData0 = Numeric.sum(Numeric.sum(stack.data, 2),1)
+        mcaData0 = numpy.sum(numpy.sum(stack.data, 2),1)
     else:
-        mcaData0 = Numeric.sum(Numeric.sum(stack.data, 2),0)
+        mcaData0 = numpy.sum(numpy.sum(stack.data, 2),0)
 
     import McaWindow
     mca = McaWindow.McaWidget()
@@ -597,9 +620,9 @@ if __name__ == "__main__":
             iy1 = int(ddict['xmin'])
             iy2 = int(ddict['xmax'])+1
             if imax == 0:
-                selectedData = Numeric.sum(Numeric.sum(stack.data[:,ix1:ix2, iy1:iy2], 2),1)
+                selectedData = numpy.sum(numpy.sum(stack.data[:,ix1:ix2, iy1:iy2], 2),1)
             else:
-                selectedData = Numeric.sum(Numeric.sum(stack.data[ix1:ix2,:, iy1:iy2], 2),0)
+                selectedData = numpy.sum(numpy.sum(stack.data[ix1:ix2,:, iy1:iy2], 2),0)
             sel = {}
             sel['SourceName'] = "EDF Stack"
             sel['Key'] = "Selection"
