@@ -1,38 +1,43 @@
 #/*##########################################################################
-# Copyright (C) 2004-2010 European Synchrotron Radiation Facility
+# Copyright (C) 2004-2012 European Synchrotron Radiation Facility
 #
-# This file is part of the PyMCA X-ray Fluorescence Toolkit developed at
-# the ESRF by the Beamline Instrumentation Software Support (BLISS) group.
+# This file is part of the PyMca X-ray Fluorescence Toolkit developed at
+# the ESRF by the Software group.
 #
-# This toolkit is free software; you can redistribute it and/or modify it 
+# This toolkit is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by the Free
-# Software Foundation; either version 2 of the License, or (at your option) 
+# Software Foundation; either version 2 of the License, or (at your option)
 # any later version.
 #
-# PyMCA is distributed in the hope that it will be useful, but WITHOUT ANY
+# PyMca is distributed in the hope that it will be useful, but WITHOUT ANY
 # WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 # FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 # details.
 #
 # You should have received a copy of the GNU General Public License along with
-# PyMCA; if not, write to the Free Software Foundation, Inc., 59 Temple Place,
-# Suite 330, Boston, MA 02111-1307, USA.
+# PyMca; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #
-# PyMCA follows the dual licensing model of Trolltech's Qt and Riverbank's PyQt
-# and cannot be used as a free plugin for a non-free program. 
+# PyMca follows the dual licensing model of Riverbank's PyQt and cannot be
+# used as a free plugin for a non-free program.
 #
-# Please contact the ESRF industrial unit (industry@esrf.fr) if this license 
+# Please contact the ESRF industrial unit (industry@esrf.fr) if this license
 # is a problem for you.
 #############################################################################*/
 import posixpath
-import DataObject
+import numpy
 import h5py
+try:
+    from PyMca import DataObject
+except ImportError:
+    print("HDF5Stack1D importing DataObject from local directory!")
+    import DataObject
 try:
     from PyMca import NexusDataSource
-except:
+except ImportError:
+    print("HDF5Stack1D importing NexusDataSource from local directory!")
     import NexusDataSource
-import posixpath
-import numpy
+
 DEBUG = 0    
 SOURCE_TYPE = "HDF5Stack1D"
 
@@ -43,7 +48,8 @@ class HDF5Stack1D(DataObject.DataObject):
         DataObject.DataObject.__init__(self)
 
         #the data type of the generated stack
-        self.__dtype = dtype
+        self.__dtype0 = dtype 
+        self.__dtype  = dtype
 
         if filelist is not None:
             if selection is not None:
@@ -66,9 +72,6 @@ class HDF5Stack1D(DataObject.DataObject):
                  That means scanlist = ["/whatever1"]
                  and               selection['y'] = "/whatever2/counts"
         """
-        if type(filelist) == type(''):
-            filelist = [filelist]
-
         # all the files in the same source
         hdfStack = NexusDataSource.NexusDataSource(filelist)
 
@@ -80,12 +83,22 @@ class HDF5Stack1D(DataObject.DataObject):
             if isinstance(tmpHdf["/"+key], h5py.Group):
                 entryNames.append(key)
 
-        #built the selection in terms of HDF terms
-        #for the time being, the x selection will be ignored but not the
-        #monitor and only one y is taken
+        # built the selection in terms of HDF terms
+        # for the time being, the only the firs item in x selection used
+        xSelection = selection['x']
+        if type(xSelection) == type([]):
+            if len(xSelection):
+                xSelection = xSelection[0]
+            else:
+                xSelection = None
+        else:
+            xSelection = None
+        # only one y is taken
         ySelection = selection['y']
         if type(ySelection) == type([]):
             ySelection = ySelection[0]
+
+        # monitor selection
         mSelection = selection['m']
         if type(mSelection) == type([]):
             if len(mSelection):
@@ -94,8 +107,16 @@ class HDF5Stack1D(DataObject.DataObject):
                 mSelection = None
         else:
             mSelection = None
+
+        # deal with the pathological case where the scanlist corresponds
+        # to a selected top level dataset
+        if len(entryNames) == 0:
+            if scanlist is not None:
+                if len(scanlist) == 1:
+                    if scanlist[0] == ySelection:
+                        scanlist = None
             
-        if scanlist is None:
+        if scanlist in [None, []]:
             #if the scanlist is None, it is assumed we are interested on all
             #the scans containing the selection, not that all the scans
             #contain the selection.
@@ -143,21 +164,24 @@ class HDF5Stack1D(DataObject.DataObject):
                     scanlist.append("")
         else:
             try:
-                number, order = map(int, scanlist[0].split("."))
+                number, order = [int(x) for x in scanlist[0].split(".")]
                 JUST_KEYS = True
             except:
                 JUST_KEYS = False
             if not JUST_KEYS:
                 for scan in scanlist:
-                    if scan not in entryNames:
-                        raise ValueError, "Entry %s not in file"
-
+                    if scan.startswith("/"):
+                        t = scan[1:]
+                    else:
+                        t = scan
+                    if t not in entryNames:
+                        raise ValueError("Entry %s not in file" % scan)
         
         nFiles = len(filelist)
         nScans = len(scanlist)
         if JUST_KEYS:
             if not nScans:
-                raise IOError, "No entry contains the required data"
+                raise IOError("No entry contains the required data")
 
         #Now is to decide the number of mca ...
         #I assume all the scans contain the same number of mca
@@ -165,10 +189,15 @@ class HDF5Stack1D(DataObject.DataObject):
             path = "/" + entryNames[int(scanlist[0].split(".")[-1])-1] + ySelection
             if mSelection is not None:
                 mpath = "/" + entryNames[int(scanlist[0].split(".")[-1])-1] + mSelection
+            if xSelection is not None:
+                xpath = "/" + entryNames[int(scanlist[0].split(".")[-1])-1] + xSelection
         else:
-            path = "/" + scanlist[0] + ySelection
+            path = scanlist[0] +  ySelection
             if mSelection is not None:
-                mpath = "/" + scanlist[0] + mSelection
+                mpath = scanlist[0] + mSelection
+            if xSelection is not None:
+                xpath = scanlist[0] + xSelection
+        
         yDataset = tmpHdf[path]
 
         if self.__dtype is None:
@@ -182,23 +211,50 @@ class HDF5Stack1D(DataObject.DataObject):
         dim0, dim1, mcaDim = self.getDimensions(nFiles, nScans, shape,
                                                 index=mcaIndex)
         try:
+            if self.__dtype in [numpy.float32, numpy.int32]:
+                bytefactor = 4
+            elif self.__dtype in [numpy.int16, numpy.uint16]:
+                bytefactor = 2
+            elif self.__dtype in [numpy.int8, numpy.uint8]:
+                bytefactor = 1
+            else:
+                bytefactor = 8
+
+            neededMegaBytes = nFiles * dim0 * dim1 * mcaDim * bytefactor/(1024*1024.)
+            if (neededMegaBytes > 3000) and (nFiles == 1) and (len(shape) == 3):
+                if self.__dtype0 is None:
+                    if (bytefactor == 8) and (neededMegaBytes <6000):
+                        #try reading as float32
+                        self.__dtype = numpy.float32
+                    else:
+                        raise MemoryError("Force dynamic loading")
+                else:
+                    raise MemoryError("Force dynamic loading")
             self.data = numpy.zeros((dim0, dim1, mcaDim), self.__dtype)
             DONE = False
-        except MemoryError:
+        except (MemoryError, ValueError):
+            #some versions report ValueError instead of MemoryError
             if (nFiles == 1) and (len(shape) == 3):
-                print "Attempting dynamic loading"
+                print("Attempting dynamic loading")
                 self.data = yDataset
                 if mSelection is not None:
-                    mDataset = tmpHdf[mpath]
-                    self.monitor = mDataset
+                    mDataset = tmpHdf[mpath].value
+                    self.monitor = [mDataset]
+                if xSelection is not None:
+                    xDataset = tmpHdf[xpath].value
+                    self.x = [xDataset]
+                if h5py.version.version < '2.0':
+                    #prevent automatic closing keeping a reference
+                    #to the open file
+                    self._fileReference = hdfStack
                 DONE = True
             else:
+                #what to do if the number of dimensions is only 2?
                 raise
         
         if not DONE:
             self.info["McaIndex"] = 2
             n = 0
-            i_idx = dim0 * dim1
 
             if dim0 == 1:
                 self.onBegin(dim1)
@@ -206,7 +262,7 @@ class HDF5Stack1D(DataObject.DataObject):
                 self.onBegin(dim0)
             self.incrProgressBar=0
             for hdf in hdfStack._sourceObjectList:
-                entryNames = hdf["/"].keys()
+                entryNames = list(hdf["/"].keys())
                 for scan in scanlist:
                     if JUST_KEYS:
                         entryName = entryNames[int(scan.split(".")[-1])-1]
@@ -214,64 +270,164 @@ class HDF5Stack1D(DataObject.DataObject):
                         if mSelection is not None:
                             mpath = entryName + mSelection
                             mDataset = hdf[mpath].value
+                        if xSelection is not None:
+                            xpath = entryName + xSelection
+                            xDataset = hdf[xpath].value
                     else:
                         path = scan + ySelection
                         if mSelection is not None:
                             mpath = scan + mSelection
                             mDataset = hdf[mpath].value
-                    yDataset = hdf[path].value
+                        if xSelection is not None:
+                            xpath = scan + xSelection
+                            xDataset = hdf[xpath].value
+                    try:
+                        yDataset = hdf[path]
+                        tmpShape = yDataset.shape
+                        totalBytes = numpy.ones((1,), yDataset.dtype).itemsize
+                        for nItems in tmpShape:
+                            totalBytes *= nItems
+                        if (totalBytes/(1024.*1024.)) > 500:
+                            #read from disk
+                            IN_MEMORY = False
+                        else:
+                            #read the data into memory
+                            yDataset = hdf[path].value 
+                            IN_MEMORY = True
+                    except (MemoryError, ValueError):
+                        yDataset = hdf[path]
+                        IN_MEMORY = False
+                    nMcaInYDataset = 1
+                    for dim in yDataset.shape:
+                        nMcaInYDataset *= dim
+                    nMcaInYDataset = int(nMcaInYDataset/mcaDim)
                     if mcaIndex != 0:
-                        yDataset.shape = -1, mcaDim
+                        if IN_MEMORY:
+                            yDataset.shape = -1, mcaDim
+                        if mSelection is not None:
+                            case = -1
+                            nMonitorData = 1
+                            for  v in mDataset.shape:
+                                nMonitorData *= v
+                            if nMonitorData == nMcaInYDataset:
+                                mDataset.shape = nMcaInYDataset
+                                case = 0
+                            elif nMonitorData == (nMcaInYDataset * mcaDim):
+                                case = 1
+                                mDataset.shape = nMcaInYDataset, mcaDim
+                            if case == -1:
+                                raise ValueError(\
+                                    "I do not know how to handle this monitor data")
+                        if (len(yDataset.shape) == 3) and\
+                           (dim1 == yDataset.shape[1]):
+                            mca = 0
+                            deltaI = int(yDataset.shape[1]/dim1)
+                            for ii in range(yDataset.shape[0]):
+                                i = int(n/dim1)
+                                yData = yDataset[ii:(ii+1)]
+                                yData.shape = -1, mcaDim
+                                if mSelection is not None:
+                                    if case == 0:
+                                        mData = numpy.outer(mDataset[mca:(mca+dim1)],
+                                                            numpy.ones((mcaDim)))
+                                        self.data[i, :, :] = yData/mData
+                                    elif case == 1:
+                                        mData = mDataset[mca:(mca+dim1), :]
+                                        mData.shape = -1, mcaDim
+                                        self.data[i, :, :]  = yData/mData
+                                else:
+                                    self.data[i:(i+deltaI), :] = yData
+                                n += yDataset.shape[1]
+                                mca += dim1
+                        else:
+                            for mca in range(nMcaInYDataset):
+                                i = int(n/dim1)
+                                j = n % dim1
+                                if len(yDataset.shape) == 3:
+                                    ii = int(mca/yDataset.shape[1])
+                                    jj = mca % yDataset.shape[1]
+                                    yData = yDataset[ii, jj]
+                                elif len(yDataset.shape) == 2:
+                                    yData = yDataset[mca,:]
+                                elif len(yDataset.shape) == 1:
+                                    yData = yDataset
+                                if mSelection is not None:
+                                    if case == 0:
+                                        self.data[i, j, :] = yData/mDataset[mca]
+                                    elif case == 1:
+                                        self.data[i, j, :]  = yData/mDataset[mca, :]
+                                else:
+                                    self.data[i, j, :] = yData
+                                n += 1
+                    else:
                         if mSelection is not None:
                             case = -1
                             nMonitorData = 1
                             for  v in mDataset.shape:
                                 nMonitorData *= v
                             if nMonitorData == yDataset.shape[0]:
+                                case = 3
                                 mDataset.shape = yDataset.shape[0]
-                                case = 0
-                            elif nMonitorData == (yDataset.shape[0] * yDataset.shape[1]):
-                                case = 1
-                                mDataset.shape = yDataset.shape[0], yDataset.shape[1]
-                            if case == -1:
-                                raise ValueError, "I do not know how to handle this monitor data"
-                        for mca in range(yDataset.shape[0]):
-                            i = int(n/dim1)
-                            j = n % dim1
-                            if mSelection is not None:
-                                if case == 0:
-                                    self.data[i, j, :] = yDataset[mca,:]/mDataset[mca]
-                                elif case == 1:
-                                    self.data[i, j, :]  = yDataset[mca,:]/mDataset[mca, :]
-                            else:
-                                self.data[i, j, :] = yDataset[mca,:]
-                            n += 1
-                    else:
-                        yDataset.shape = mcaDim, -1
-                        if mSelection is not None:
-                            case = -1
-                            nMonitorData = 1
-                            for  v in mDataset.shape:
-                                nMonitorData *= v
-                            if nMonitorData == yDataset.shape[1]:
-                                mDataset.shape = yDataset.shape[1]
+                            elif nMonitorData == nMcaInYDataset:
+                                mDataset.shape = nMcaInYDataset
                                 case = 0
                             #elif nMonitorData == (yDataset.shape[1] * yDataset.shape[2]):
                             #    case = 1
                             #    mDataset.shape = yDataset.shape[1], yDataset.shape[2]
                             if case == -1:
-                                raise ValueError, "I do not know how to handle this monitor data"
-                        for mca in range(yDataset.shape[1]):
-                            i = int(n/dim1)
-                            j = n % dim1
+                                raise ValueError(\
+                                    "I do not know how to handle this monitor data")
+                        if IN_MEMORY:
+                            yDataset.shape = mcaDim, -1
+                        if len(yDataset.shape) != 3:
+                            for mca in range(nMcaInYDataset):
+                                i = int(n/dim1)
+                                j = n % dim1
+                                if len(yDataset.shape) == 3:
+                                    ii = int(mca/yDataset.shape[2])
+                                    jj = mca % yDataset.shape[2]
+                                    yData = yDataset[:, ii, jj]
+                                elif len(yDataset.shape) == 2:
+                                    yData = yDataset[:, mca]
+                                elif len(yDataset.shape) == 1:
+                                    yData = yDataset[:]                            
+                                if mSelection is not None:
+                                    if case == 0:
+                                        self.data[i, j, :] = yData/mDataset[mca]
+                                    elif case == 1:
+                                        self.data[i, j, :]  = yData/mDataset[:, mca]
+                                    elif case == 3:
+                                        self.data[i, j, :]  = yData/mDataset
+                                else:
+                                    self.data[i, j, :] = yData
+                                n += 1
+                        else:
+                            #stack of images to be read as MCA
+                            for nImage in range(yDataset.shape[0]):
+                                tmp = yDataset[nImage:(nImage+1)]
+                                if len(tmp.shape) == 3:
+                                    i = int(n/dim1)
+                                    j = n % dim1
+                                    if 0:
+                                        #this loop is extremely SLOW!!!(and useless)
+                                        for ii in range(tmp.shape[1]):
+                                            for jj in range(tmp.shape[2]):
+                                                self.data[i+ii, j+jj, nImage] = tmp[0, ii, jj]
+                                    else:
+                                        self.data[i:i+tmp.shape[1],
+                                                  j:j+tmp.shape[2], nImage] = tmp[0]
                             if mSelection is not None:
-                                if case == 0:
-                                    self.data[i, j, :] = yDataset[:,mca]/mDataset[mca]
-                                elif case == 1:
-                                    self.data[i, j, :]  = yDataset[:, mca]/mDataset[:, mca]
+                                for mca in range(yDataset.shape[0]):
+                                    i = int(n/dim1)
+                                    j = n % dim1
+                                    yData = self.data[i, j, :]
+                                    if case == 0:
+                                        self.data[i, j, :] = yData/mDataset[mca]
+                                    elif case == 1:
+                                        self.data[i, j, :]  = yData/mDataset[:, mca]
+                                    n += 1
                             else:
-                                self.data[i, j, :] = yDataset[:, mca]
-                            n += 1
+                                n += tmp.shape[1] * tmp.shape[2]
                     if dim0 == 1:
                         self.onProgress(j)
                 if dim0 != 1:
@@ -295,6 +451,11 @@ class HDF5Stack1D(DataObject.DataObject):
         for i in range(len(shape)):
             key = 'Dim_%d' % (i+1,)
             self.info[key] = shape[i]
+        if xSelection is not None:
+            if xDataset.size == shape[self.info['McaIndex']]:
+                self.x = [xDataset.reshape(-1)]
+            else:
+                print("Ignoring xSelection")
 
 
     def getDimensions(self, nFiles, nScans, shape, index=None):
@@ -307,11 +468,11 @@ class HDF5Stack1D(DataObject.DataObject):
         if index == -1:
             index = len(shape) - 1
         if DEBUG:
-            print "INDEX = ", index
+            print("INDEX = %d" % index)
         #figure out the shape of the stack
         if len(shape) == 0:
             #a scalar?
-            raise ValueError, "Selection corresponds to a scalar"
+            raise ValueError("Selection corresponds to a scalar")
         elif len(shape) == 1:
             #nchannels
             nMca = 1
@@ -329,7 +490,7 @@ class HDF5Stack1D(DataObject.DataObject):
             elif index == 0:
                 nMca = shape[1] * shape[2]
             else:
-                raise IndexError, "Only first and last dimensions handled"
+                raise IndexError("Only first and last dimensions handled")
         else:
             nMca = 1
             for i in range(len(shape)):
@@ -339,8 +500,8 @@ class HDF5Stack1D(DataObject.DataObject):
                 
         mcaDim = shape[index]
         if DEBUG:
-            print "nMca = ", nMca
-            print "mcaDim = ", mcaDim
+            print("nMca = %d" % nMca)
+            print("mcaDim = ", mcaDim)
 
         # HDF allows to work directly from the files without loading
         # them into memory.
@@ -402,7 +563,7 @@ class HDF5Stack1D(DataObject.DataObject):
             dim1 = nMca * nScans
         else:
             #I should not reach this point
-            raise ValueError, "Unhandled case"
+            raise ValueError("Unhandled case")
 
         return dim0, dim1, shape[index]
 
@@ -414,6 +575,3 @@ class HDF5Stack1D(DataObject.DataObject):
 
     def onEnd(self):
         pass
-
-if __name__ == "__main__":
-    import sys
