@@ -1,33 +1,90 @@
 """
 """
 from __future__ import with_statement
-import operator
+import sys
 import os
-import shutil
 import posixpath
 import gc
-
-import PyQt4.QtCore as QtCore
-import PyQt4.QtGui as QtGui
+import re
+from operator import itemgetter
 
 try:
-    import PyMca.phynx as phynx
-except:
-    #I should never reach here
-    try:
-        from xpaxs.io import phynx
-    except ImportError:
-        import phynx
+    from PyMca import PyMcaQt as qt
+    safe_str = qt.safe_str
+except ImportError:
+    # for people using this widget without PyMca installed
+    import PyQt4.Qt as qt
+    safe_str = str
 
+if hasattr(qt, 'QStringList'):
+    MyQVariant = qt.QVariant
+else:
+    def MyQVariant(x=None):
+        return x
+
+import h5py
+phynx = h5py
 import weakref
 
-class QRLock(QtCore.QMutex):
+DEBUG = 0
+
+#sorting method
+def h5py_sorting(object_list):
+    sorting_list = ['start_time', 'end_time', 'name']
+    n = len(object_list)
+    if n < 2:
+        return object_list
+
+    # This implementation only sorts entries
+    if posixpath.dirname(object_list[0].name) != "/":
+        return object_list
+
+    names = list(object_list[0].keys())
+
+    sorting_key = None
+    for key in sorting_list:
+        if key in names:
+            sorting_key = key
+            break
+        
+    if sorting_key is None:
+        if 'name' in sorting_list:
+            sorting_key = 'name'
+        else:
+            return object_list
+
+    try:
+        if sorting_key != 'name':
+            sorting_list = [(o[sorting_key].value, o)
+                           for o in object_list]
+            sorted_list = sorted(sorting_list, key=itemgetter(0))
+            return [x[1] for x in sorted_list]
+
+        if sorting_key == 'name':
+            sorting_list = [(_get_number_list(o.name),o)
+                           for o in object_list]
+            sorting_list.sort()
+            return [x[1] for x in sorting_list]
+    except:
+        #The only way to reach this point is to have different
+        #structures among the different entries. In that case
+        #defaults to the unfiltered case
+        print("WARNING: Default ordering")
+        print("Probably all entries do not have the key %s" % sorting_key)
+        return object_list
+
+def _get_number_list(txt):
+    rexpr = '[/a-zA-Z:-]'
+    nbs= [float(w) for w in re.split(rexpr, txt) if w not in ['',' ']]
+    return nbs
+
+class QRLock(qt.QMutex):
 
     """
     """
 
     def __init__(self):
-        QtCore.QMutex.__init__(self, QtCore.QMutex.Recursive)
+        qt.QMutex.__init__(self, qt.QMutex.Recursive)
 
     def __enter__(self):
         self.lock()
@@ -90,9 +147,18 @@ class H5NodeProxy(object):
         if not self._children:
             # obtaining the lock here is necessary, otherwise application can
             # freeze if navigating tree while data is processing
-            with self.file.plock:
+            if 1: #with self.file.plock:
+                tmpList = list(self.getNode(self.name).values())
+                try:
+                    finalList = h5py_sorting(tmpList)
+                except:
+                    #one cannot afford any error
+                    if DEBUG:
+                        raise
+                    else:
+                        finalList = tmpList
                 self._children = [H5NodeProxy(self.file, i, self)
-                                  for i in self.getNode(self.name).values()]
+                                  for i in finalList]
         return self._children
 
     @property
@@ -109,7 +175,7 @@ class H5NodeProxy(object):
 
     @property
     def row(self):
-        with self.file.plock:
+        if 1:#with self.file.plock:
             try:
                 return self.parent.children.index(self)
             except ValueError:
@@ -142,7 +208,7 @@ class H5NodeProxy(object):
 
 
     def __init__(self, ffile, node, parent=None):
-        with ffile.plock:
+        if 1:#with ffile.plock:
             self._file = ffile
             self._parent = parent
             self._name = node.name
@@ -153,15 +219,26 @@ class H5NodeProxy(object):
                 self._name = posixpath.basename(node.name)
             """
             self._type = type(node).__name__
-            self._hasChildren = isinstance(node, phynx.Group)
+            self._hasChildren = isinstance(node, h5py.Group)
+            if hasattr(node, 'attrs'):
+                attrs = list(node.attrs)
+                for cname in ['class', 'NX_class']:
+                    if cname in attrs:
+                        if sys.version <'3.0':
+                            _type = "%s" % node.attrs[cname]
+                        else:
+                            _type = node.attrs[cname].decode('utf=8')
+                        self._type = _type
+                        break
+                        #self._type = _type[2].upper() + _type[3:]
             self._children = []
             if hasattr(node, 'dtype'):
-                self._dtype = str(node.dtype)
+                self._dtype = safe_str(node.dtype)
             else:
                 self._dtype = ""
             if hasattr(node, 'shape'):
                 if 0:
-                    self._shape = str(node.shape)
+                    self._shape = safe_str(node.shape)
                 else:
                     self._shape = node.shape
             else:
@@ -206,15 +283,15 @@ class H5FileProxy(H5NodeProxy):
             return H5NodeProxy(self.file, self.file[path], self)
 
 
-class FileModel(QtCore.QAbstractItemModel):
+class FileModel(qt.QAbstractItemModel):
 
     """
     """
 
     def __init__(self, parent=None):
-        QtCore.QAbstractItemModel.__init__(self, parent)
+        qt.QAbstractItemModel.__init__(self, parent)
         self.rootItem = RootItem(['File/Group/Dataset', 'Description', 'Shape', 'DType'])
-        self._idMap = {QtCore.QModelIndex().internalId(): self.rootItem}
+        self._idMap = {qt.QModelIndex().internalId(): self.rootItem}
 
     def clearRows(self, index):
         self.getProxyFromIndex(index).clearChildren()
@@ -228,45 +305,37 @@ class FileModel(QtCore.QAbstractItemModel):
         return 4
 
     def data(self, index, role):
-        if role != QtCore.Qt.DisplayRole:
-            return QtCore.QVariant()
+        if role != qt.Qt.DisplayRole:
+            return MyQVariant()
         item = self.getProxyFromIndex(index)
         column = index.column()
         if column == 0:
-            """
-            # I would expect the title to be the proper
-            # thing to show ...
-            if hasattr(item, 'type'):
-                if item.type == "Entry":
-                    children = item.children
-                    names = [posixpath.basename(o.name) for o in children]
-                    if "title" in names:
-                        idx = names.index("title")
-                        children[idx].getNode().value[0]
-                        return QtCore.QVariant(children[idx].getNode().value[0])
-            """
             if isinstance(item, H5FileProxy):
-                return QtCore.QVariant(os.path.basename(item.file.filename))
+                return MyQVariant(os.path.basename(item.file.filename))
             else:
-                showtitle = False
-                if showtitle:
-                    if hasattr(item, 'type'):
-                        if item.type == "Entry":
-                            children = item.children
-                            names = [posixpath.basename(o.name) for o in children]
-                            if "title" in names:
-                                idx = names.index("title")
-                                children[idx].getNode().value[0]
-                                return QtCore.QVariant(children[idx].getNode().value[0])
-                    return QtCore.QVariant(posixpath.basename(item.name))
-                return QtCore.QVariant(posixpath.basename(item.name))
+                return MyQVariant(posixpath.basename(item.name))
         if column == 1:
-            return QtCore.QVariant(item.type)
+            showtitle = True
+            if showtitle:
+                if hasattr(item, 'type'):
+                    if item.type in ["Entry", "NXentry"]:
+                        children = item.children
+                        names = [posixpath.basename(o.name) for o in children]
+                        if "title" in names:
+                            idx = names.index("title")
+                            if len(children[idx].getNode().shape):
+                                #stored as an array of strings!!!
+                                #return just the first item
+                                return MyQVariant("%s" % children[idx].getNode().value[0])
+                            else:
+                                #stored as a string
+                                return MyQVariant("%s" % children[idx].getNode().value)
+            return MyQVariant(item.type)
         if column == 2:
-            return QtCore.QVariant(item.shape)
+            return MyQVariant(item.shape)
         if column == 3:
-            return QtCore.QVariant(item.dtype)
-        return QtCore.QVariant()
+            return MyQVariant(item.dtype)
+        return MyQVariant()
 
     def getNodeFromIndex(self, index):
         try:
@@ -288,11 +357,11 @@ class FileModel(QtCore.QAbstractItemModel):
         return self.getProxyFromIndex(index).hasChildren
 
     def headerData(self, section, orientation, role):
-        if orientation == QtCore.Qt.Horizontal and \
-                role == QtCore.Qt.DisplayRole:
-            return QtCore.QVariant(self.rootItem.header[section])
+        if orientation == qt.Qt.Horizontal and \
+                role == qt.Qt.DisplayRole:
+            return MyQVariant(self.rootItem.header[section])
 
-        return QtCore.QVariant()
+        return MyQVariant()
 
     def hasIndex(self, row, column, parent):
         parentItem = self.getProxyFromIndex(parent)
@@ -303,7 +372,7 @@ class FileModel(QtCore.QAbstractItemModel):
     def index(self, row, column, parent):
         parentItem = self.getProxyFromIndex(parent)
         if row >= len(parentItem.children):
-            return QtCore.QModelIndex()
+            return qt.QModelIndex()
         child = parentItem.children[row]
         #force a pointer to child and not use id(child)
         index = self.createIndex(row, column, child)
@@ -314,11 +383,11 @@ class FileModel(QtCore.QAbstractItemModel):
         child = self.getProxyFromIndex(index)
         parent = child.parent
         if parent == self.rootItem:
-            return QtCore.QModelIndex()
+            return qt.QModelIndex()
         if parent is None:
-            return QtCore.QModelIndex()
+            return qt.QModelIndex()
         if parent.row is None:
-            return QtCore.QModelIndex()
+            return qt.QModelIndex()
         else:
             return self.createIndex(parent.row, 0, parent)
 
@@ -332,10 +401,9 @@ class FileModel(QtCore.QAbstractItemModel):
                 ddict = {}
                 ddict['event'] = "fileUpdated"
                 ddict['filename'] = filename
-                self.emit(QtCore.SIGNAL('fileUpdated'), ddict)
+                self.emit(qt.SIGNAL('fileUpdated'), ddict)
                 return item.file
-        #phynxFile = phynx.File(filename, 'a', lock=QRLock())
-        phynxFile = phynx.File(filename, 'r', lock=QRLock())
+        phynxFile = phynx.File(filename, 'r')
         if weakreference:
             def phynxFileInstanceDistroyed(weakrefObject):
                 idx = self.rootItem._identifiers.index(id(weakrefObject))
@@ -353,7 +421,7 @@ class FileModel(QtCore.QAbstractItemModel):
         ddict = {}
         ddict['event'] = "fileAppended"
         ddict['filename'] = filename
-        self.emit(QtCore.SIGNAL('fileAppended'), ddict)
+        self.emit(qt.SIGNAL('fileAppended'), ddict)
         return phynxFile
 
     def appendPhynxFile(self, phynxFile, weakreference=True):
@@ -377,7 +445,7 @@ class FileModel(QtCore.QAbstractItemModel):
             ddict = {}
             ddict['event'] = "fileUpdated"
             ddict['filename'] = name
-            self.emit(QtCore.SIGNAL('fileUpdated'), ddict)
+            self.emit(qt.SIGNAL('fileUpdated'), ddict)
             return
             
         if weakreference:
@@ -397,33 +465,33 @@ class FileModel(QtCore.QAbstractItemModel):
         ddict = {}
         ddict['event'] = "fileAppended"
         ddict['filename'] = name
-        self.emit(QtCore.SIGNAL('fileAppended'), ddict)
+        self.emit(qt.SIGNAL('fileAppended'), ddict)
 
     def clear(self):
         self.reset()
 
 
-class FileView(QtGui.QTreeView):
+class FileView(qt.QTreeView):
     def __init__(self, fileModel, parent=None):
-        QtGui.QTreeView.__init__(self, parent)
+        qt.QTreeView.__init__(self, parent)
         self.setModel(fileModel)
         self.setColumnWidth(0, 250)
         #This removes the children after a double click
         #with no possibility to recover them
         #self.connect(
         #    self,
-        #    QtCore.SIGNAL('collapsed(QModelIndex)'),
+        #    qt.SIGNAL('collapsed(QModelIndex)'),
         #    fileModel.clearRows
         #)
         self.connect(
             fileModel,
-            QtCore.SIGNAL('fileAppended'),
+            qt.SIGNAL('fileAppended'),
             self.fileAppended
         )
 
         self.connect(
             fileModel,
-            QtCore.SIGNAL('fileUpdated'),
+            qt.SIGNAL('fileUpdated'),
             self.fileUpdated
         )
 
@@ -444,38 +512,38 @@ class FileView(QtGui.QTreeView):
                     item = self.model().getProxyFromIndex(modelIndex)
                     if item.name == ddict['filename']:            
                         self.selectionModel().setCurrentIndex(modelIndex,
-                                              QtGui.QItemSelectionModel.NoUpdate)
+                                              qt.QItemSelectionModel.NoUpdate)
                         self.scrollTo(modelIndex,
-                                      QtGui.QAbstractItemView.PositionAtTop)
+                                      qt.QAbstractItemView.PositionAtTop)
                         break
         self.doItemsLayout()
                     
 class HDF5Widget(FileView):
     def __init__(self, model, parent=None):
         FileView.__init__(self, model, parent)
-        self.setSelectionBehavior(QtGui.QAbstractItemView.SelectRows)
+        self.setSelectionBehavior(qt.QAbstractItemView.SelectRows)
         self._adjust()
         if 0:
-            QtCore.QObject.connect(self,
-                     QtCore.SIGNAL('activated(QModelIndex)'),
+            qt.QObject.connect(self,
+                     qt.SIGNAL('activated(QModelIndex)'),
                      self.itemActivated)
 
-        QtCore.QObject.connect(self,
-                     QtCore.SIGNAL('clicked(QModelIndex)'),
+        qt.QObject.connect(self,
+                     qt.SIGNAL('clicked(QModelIndex)'),
                      self.itemClicked)
 
-        QtCore.QObject.connect(self,
-                     QtCore.SIGNAL('doubleClicked(QModelIndex)'),
+        qt.QObject.connect(self,
+                     qt.SIGNAL('doubleClicked(QModelIndex)'),
                      self.itemDoubleClicked)
 
-        QtCore.QObject.connect(
+        qt.QObject.connect(
             self,
-            QtCore.SIGNAL('collapsed(QModelIndex)'),
+            qt.SIGNAL('collapsed(QModelIndex)'),
             self._adjust)
 
-        QtCore.QObject.connect(
+        qt.QObject.connect(
             self,
-            QtCore.SIGNAL('expanded(QModelIndex)'),
+            qt.SIGNAL('expanded(QModelIndex)'),
             self._adjust)
 
     def _adjust(self, modelIndex=None):
@@ -486,16 +554,16 @@ class HDF5Widget(FileView):
         
     def mousePressEvent(self, e):
         button = e.button()
-        if button == QtCore.Qt.LeftButton:
+        if button == qt.Qt.LeftButton:
             self._lastMouse = "left"
-        elif button == QtCore.Qt.RightButton:
+        elif button == qt.Qt.RightButton:
             self._lastMouse = "right"
-        elif button == QtCore.Qt.MidButton:
+        elif button == qt.Qt.MidButton:
             self._lastMouse = "middle"
         else:
             #Should I set it to no button?
             self._lastMouse = "left"
-        QtGui.QTreeView.mousePressEvent(self, e)
+        qt.QTreeView.mousePressEvent(self, e)
 
     def itemActivated(self, modelIndex):
         event ="itemActivated"
@@ -521,7 +589,7 @@ class HDF5Widget(FileView):
         ddict['dtype'] = item.dtype
         ddict['shape'] = item.shape
         ddict['mouse'] = self._lastMouse * 1
-        self.emit(QtCore.SIGNAL("HDF5WidgetSignal"), ddict)
+        self.emit(qt.SIGNAL("HDF5WidgetSignal"), ddict)
 
     def getSelectedEntries(self):
         modelIndexList = self.selectedIndexes()
@@ -543,16 +611,19 @@ class HDF5Widget(FileView):
 
 
 if __name__ == "__main__":
-    import sys
-    app = QtGui.QApplication(sys.argv)
+    app = qt.QApplication(sys.argv)
+    if len(sys.argv) < 2:
+        print("Usage:")
+        print("python HDF5Widget.py path_to_hdf5_file_name")
+        sys.exit(0)
     fileModel = FileModel()
     fileView = HDF5Widget(fileModel)
     #fileModel.openFile('/home/darren/temp/PSI.hdf')
     phynxFile = fileModel.openFile(sys.argv[1])
     def mySlot(ddict):
-        print ddict
+        print(ddict)
         if ddict['type'].lower() in ['dataset']:
-            print phynxFile[ddict['name']].dtype, phynxFile[ddict['name']].shape
-    QtCore.QObject.connect(fileView, QtCore.SIGNAL("HDF5WidgetSignal"), mySlot)
+            print(phynxFile[ddict['name']].dtype, phynxFile[ddict['name']].shape)
+    qt.QObject.connect(fileView, qt.SIGNAL("HDF5WidgetSignal"), mySlot)
     fileView.show()
     sys.exit(app.exec_())
