@@ -1,61 +1,62 @@
 #/*##########################################################################
-# Copyright (C) 2004-2010 European Synchrotron Radiation Facility
+# Copyright (C) 2004-2012 European Synchrotron Radiation Facility
 #
-# This file is part of the PyMCA X-ray Fluorescence Toolkit developed at
-# the ESRF by the Beamline Instrumentation Software Support (BLISS) group.
+# This file is part of the PyMca X-ray Fluorescence Toolkit developed at
+# the ESRF by the Software group.
 #
-# This toolkit is free software; you can redistribute it and/or modify it 
+# This toolkit is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by the Free
-# Software Foundation; either version 2 of the License, or (at your option) 
+# Software Foundation; either version 2 of the License, or (at your option)
 # any later version.
 #
-# PyMCA is distributed in the hope that it will be useful, but WITHOUT ANY
+# PyMca is distributed in the hope that it will be useful, but WITHOUT ANY
 # WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 # FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 # details.
 #
 # You should have received a copy of the GNU General Public License along with
-# PyMCA; if not, write to the Free Software Foundation, Inc., 59 Temple Place,
-# Suite 330, Boston, MA 02111-1307, USA.
+# PyMca; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #
-# PyMCA follows the dual licensing model of Trolltech's Qt and Riverbank's PyQt
-# and cannot be used as a free plugin for a non-free program. 
+# PyMca follows the dual licensing model of Riverbank's PyQt and cannot be
+# used as a free plugin for a non-free program.
 #
-# Please contact the ESRF industrial unit (industry@esrf.fr) if this license 
+# Please contact the ESRF industrial unit (industry@esrf.fr) if this license
 # is a problem for you.
 #############################################################################*/
 __revision__ = "$Revision: 1.32 $"
-import ClassMcaTheory
-import SpecFileLayer
-import EdfFileLayer
-import EdfFile
-import LuciaMap
-import AifiraMap
-import EDFStack
+import sys
+import os
+import numpy
+from PyMca import ClassMcaTheory
+from PyMca import SpecFileLayer
+from PyMca import EdfFileLayer
+from PyMca import EdfFile
+from PyMca import LuciaMap
+from PyMca import AifiraMap
+from PyMca import EDFStack
 try:
     import h5py
-    import HDF5Stack1D
+    from PyMca import HDF5Stack1D
     HDF5SUPPORT = True
 except ImportError:
     HDF5SUPPORT = False
-import numpy
-import numpy.oldnumeric as Numeric
-import os
-import sys
-import ConfigDict
-import ConcentrationsTool
+from PyMca import ConfigDict
+from PyMca import ConcentrationsTool
 
 
-class McaAdvancedFitBatch:
+class McaAdvancedFitBatch(object):
     def __init__(self,initdict,filelist=None,outputdir=None,
                     roifit=None,roiwidth=None,
                     overwrite=1, filestep=1, mcastep=1,
                     concentrations=0, fitfiles=1, fitimages=1,
                     filebeginoffset = 0, fileendoffset=0,
-                    mcaoffset=0, chunk = None, selection=None):
+                    mcaoffset=0, chunk = None,
+                    selection=None, lock=None):
         #for the time being the concentrations are bound to the .fit files
         #that is not necessary, but it will be correctly implemented in
         #future releases
+        self._lock = lock
         self.fitFiles = fitfiles
         self._concentrations = concentrations
         if type(initdict) == type([]):
@@ -163,8 +164,9 @@ class McaAdvancedFitBatch:
             if self.__stack is None:
                 self.__stack = False
                 if hasattr(self.file, "info"):
-                    if self.file.info.has_key("SourceType"):
-                        if self.file.info["SourceType"] in ["EdfFileStack", "HDF5Stack1D"]:
+                    if "SourceType" in self.file.info:
+                        if self.file.info["SourceType"] in\
+                           ["EdfFileStack", "HDF5Stack1D"]:
                             self.__stack = True
             if self.__stack:
                 self.__processStack()
@@ -185,7 +187,10 @@ class McaAdvancedFitBatch:
             if HDF5SUPPORT:
                 if h5py.is_hdf5(inputfile):
                     self._HDF5 = True
-                    return HDF5Stack1D.HDF5Stack1D(inputfile, self.selection)
+                    try:
+                        return HDF5Stack1D.HDF5Stack1D([inputfile], self.selection)
+                    except:
+                        raise
             ffile = self.__tryEdf(inputfile)
             if ffile is None:
                 ffile = self.__tryLucia(inputfile)
@@ -198,8 +203,8 @@ class McaAdvancedFitBatch:
                 ffile.SetSource(inputfile)
             return ffile
         except:
-            raise IOError, "I do not know what to do with file %s" % inputfile        
-    
+            raise IOError("I do not know what to do with file %s" % inputfile)
+
     
     def onNewFile(self,ffile, filelist):
         self.__log(ffile)
@@ -215,8 +220,8 @@ class McaAdvancedFitBatch:
         pass
             
     def __log(self,text):
-        print text
-            
+        print(text)
+  
     def __tryEdf(self,inputfile):
         try:
             ffile   = EdfFileLayer.EdfFileLayer(fastedf=0)
@@ -275,32 +280,33 @@ class McaAdvancedFitBatch:
             keylist[i] = "1.%04d" % i
 
         for i in range(nimages):
+            if self.pleaseBreak: break
             self.onImage(keylist[i], keylist)
-            self.__ncols = len(range(0+self.mcaOffset,
+            self.__ncols = numberofmca
+            colsToIter = range(0+self.mcaOffset,
                                      numberofmca,
-                                     self.mcaStep))
+                                     self.mcaStep)
             self.__row = i
             self.__col = -1
             try:
                 cache_data = data[i, :, :]
             except:
-                print "Error reading dataset row %d" % i
-                print sys.exc_info()
-                print "Batch resumed"
+                print("Error reading dataset row %d" % i)
+                print(sys.exc_info())
+                print("Batch resumed")
                 continue
-            for mca_index in range(self.__ncols):
-                mca = 0 + self.mcaOffset + mca_index * self.mcaStep
+            for mca in colsToIter:
                 if self.pleaseBreak: break
-                self.__col += 1
+                self.__col = mca
                 mcadata = cache_data[mca, :]
-                if info.has_key('MCA start ch'):
+                if 'MCA start ch' in info:
                     xmin = float(info['MCA start ch'])
                 else:
                     xmin = 0.0
                 #key = "%s.%s.%02d.%02d" % (scan,order,row,col)
                 key = "%s.%04d" % (keylist[i], mca)
-                y0  = Numeric.array(mcadata)
-                x = Numeric.arange(len(y0))*1.0 + xmin
+                y0  = numpy.array(mcadata)
+                x = numpy.arange(len(y0))*1.0 + xmin
                 #I only process the first file of the stack?
                 filename = os.path.basename(info['SourceName'][0])
                 infoDict = {}
@@ -314,18 +320,13 @@ class McaAdvancedFitBatch:
     def __processOneFile(self):
         ffile=self.file
         fileinfo = ffile.GetSourceInfo()
-        nimages = nscans = len(fileinfo['KeyList'])
         if 1:
             i = 0
             for scankey in  fileinfo['KeyList']:
                 if self.pleaseBreak: break
                 self.onImage(scankey, fileinfo['KeyList'])
-                if 0:
-                    scan,rc   = string.split(scankey,".")
-                    info,data  = ffile.LoadSource({'Key':int(image)-1})
-                else:
-                    scan,order = scankey.split(".")
-                    info,data  = ffile.LoadSource(scankey)
+                scan,order = scankey.split(".")
+                info,data  = ffile.LoadSource(scankey)
                 if info['SourceType'] == "EdfFile":
                     nrows = int(info['Dim_1'])
                     ncols = int(info['Dim_2'])
@@ -337,26 +338,16 @@ class McaAdvancedFitBatch:
                         if self.pleaseBreak: break
                         self.__col += 1
                         if int(nrows) > int(ncols):
-                            row=mca
-                            col=0
                             mcadata = data[mca,:]
                         else:
-                            col=mca
-                            row=0
                             mcadata = data[:,mca]
-                        if info.has_key('MCA start ch'):
+                        if 'MCA start ch' in info:
                             xmin = float(info['MCA start ch'])
                         else:
                             xmin = 0.0
-                        #key = "%s.%s.%02d.%02d" % (scan,order,row,col)
                         key = "%s.%s.%04d" % (scan,order,mca)
-                        if 0:
-                            #slow
-                            y0  = Numeric.array(mcadata.tolist())
-                        else:
-                            #fast
-                            y0  = Numeric.array(mcadata)
-                        x = Numeric.arange(len(y0))*1.0 + xmin
+                        y0  = numpy.array(mcadata)
+                        x = numpy.arange(len(y0))*1.0 + xmin
                         filename = os.path.basename(info['SourceName'])
                         infoDict = {}
                         infoDict['SourceName'] = info['SourceName']
@@ -380,7 +371,7 @@ class McaAdvancedFitBatch:
                         #situations
                         #if self.__row == 0:
                         if self.counter == 0:
-                            self.__chann0List = Numeric.zeros(info['NbMcaDet'])
+                            self.__chann0List = numpy.zeros(info['NbMcaDet'])
                             chan0list = scan_obj.header('@CHANN')
                             if len(chan0list):
                                 for i in range(info['NbMcaDet']):
@@ -397,8 +388,8 @@ class McaAdvancedFitBatch:
                             #get rid of slow info reading methods
                             #mcainfo,mcadata = ffile.LoadSource(key)
                             mcadata = scan_obj.mca(i+1)
-                            y0  = Numeric.array(mcadata)
-                            x = Numeric.arange(len(y0))*1.0 + \
+                            y0  = numpy.array(mcadata)
+                            x = numpy.arange(len(y0))*1.0 + \
                                 self.__chann0List[mca-1]
                             filename = os.path.basename(info['SourceName'])
 
@@ -412,26 +403,46 @@ class McaAdvancedFitBatch:
                             #print "remaining = ",(time.time()-e0) * (info['NbMca'] - i)
 
     def __getFitFile(self, filename, key):
-        fitdir = os.path.join(self._outputdir,"FIT")
-        fitdir = os.path.join(fitdir,filename+"_FITDIR")
+        fitdir = self.os_path_join(self._outputdir,"FIT")
+        fitdir = self.os_path_join(fitdir,filename+"_FITDIR")
         outfile = filename +"_"+key+".fit" 
-        outfile = os.path.join(fitdir,  outfile)           
+        outfile = self.os_path_join(fitdir,  outfile)           
+        return outfile
+
+    def os_path_join(self, a, b):
+        try:
+            outfile=os.path.join(a, b)
+        except UnicodeDecodeError:
+            toBeDone = True
+            if sys.platform == 'win32':
+                try:
+                    outfile=os.path.join(a.decode('mbcs'),
+                                         b.decode('mbcs'))
+                    toBeDone = False
+                except UnicodeDecodeError:
+                    pass
+            if toBeDone:
+                try:
+                    outfile = os.path.join(a.decode('utf-8'),
+                                           a.decode('utf-8'))
+                except UnicodeDecodeError:
+                    outfile = os.path.join(a.decode('latin-1'),
+                                           a.decode('latin-1'))
         return outfile
 
-                
     def __processOneMca(self,x,y,filename,key,info=None):
         self._concentrationsAsAscii = ""
         if not self.roiFit:
             result = None
             concentrationsdone = 0
             concentrations = None
-            outfile=os.path.join(self._outputdir,filename)
+            outfile=self.os_path_join(self._outputdir, filename)
             fitfile = self.__getFitFile(filename,key)
             if self.chunk is not None:
                 con_extension = "_%06d_partial_concentrations.txt" % self.chunk
             else:
                 con_extension = "_concentrations.txt"
-            self._concentrationsFile = os.path.join(self._outputdir,
+            self._concentrationsFile = self.os_path_join(self._outputdir,
                                     self._rootname+ con_extension)
             #                        self._rootname+"_concentrationsNEW.txt")
             if self.counter == 0:
@@ -439,7 +450,8 @@ class McaAdvancedFitBatch:
                     try:
                         os.remove(self._concentrationsFile)
                     except:
-                        print "I could not delete existing concentrations file %s", self._concentrationsFile                        
+                        print("I could not delete existing concentrations file %s" %\
+                              self._concentrationsFile)
             #print "self._concentrationsFile", self._concentrationsFile
             if self.useExistingFiles and os.path.exists(fitfile):
                 useExistingResult = 1
@@ -447,12 +459,12 @@ class McaAdvancedFitBatch:
                     dict = ConfigDict.ConfigDict()
                     dict.read(fitfile)
                     result = dict['result']
-                    if dict.has_key('concentrations'):
+                    if 'concentrations' in dict:
                         concentrationsdone = 1
                 except:
-                    print "Error trying to use result file %s" % fitfile
-                    print "Please, consider deleting it."
-                    print sys.exc_info()
+                    print("Error trying to use result file %s" % fitfile)
+                    print("Please, consider deleting it.")
+                    print(sys.exc_info())
                     return
             else:
                 useExistingResult = 0
@@ -461,7 +473,8 @@ class McaAdvancedFitBatch:
                     self.mcafit.config['fit']['use_limit'] = 1
                     self.mcafit.setdata(x,y)
                 except:
-                    print "Error entering data of file with output = ",filename
+                    print("Error entering data of file with output = %s" %\
+                          filename)
                     return
                 try:
                     self.mcafit.estimate()
@@ -478,7 +491,7 @@ class McaAdvancedFitBatch:
                             fitresult0['result']['config'] = self.mcafit.config
                             conf = self.mcafit.configure()
                             tconf = self._tool.configure()
-                            if conf.has_key('concentrations'):
+                            if 'concentrations' in conf:
                                 tconf.update(conf['concentrations'])
                             else:
                                 #what to do?
@@ -488,18 +501,19 @@ class McaAdvancedFitBatch:
                                             elementsfrommatrix=False,
                                             fluorates = self.mcafit._fluoRates)
                         except:
-                            print "error in concentrations"
-                            print sys.exc_info()[0:-1]
+                            print("error in concentrations")
+                            print(sys.exc_info()[0:-1])
                         concentrationsdone = True
                     else:
                         #just images
                         fitresult = self.mcafit.startfit(digest=0)
                 except:
-                    print "Error fitting file with output = ",filename
+                    print("Error fitting file with output = %s: %s)" %\
+                          (filename, sys.exc_info()[1]))
                     return
             if self._concentrations:
                 if concentrationsdone == 0:
-                    if not result.has_key('concentrations'):
+                    if not ('concentrations' in result):
                         if useExistingResult:
                             fitresult0={}
                             fitresult0['result'] = result
@@ -512,7 +526,7 @@ class McaAdvancedFitBatch:
                             fitresult0['fitresult'] = fitresult
                             conf = self.mcafit.configure()
                         tconf = self._tool.configure()
-                        if conf.has_key('concentrations'):
+                        if 'concentrations' in conf:
                             tconf.update(conf['concentrations'])
                         else:
                             pass
@@ -524,8 +538,8 @@ class McaAdvancedFitBatch:
                                             fitresult=fitresult0,
                                             elementsfrommatrix=False)
                         except:
-                            print "error in concentrations"
-                            print sys.exc_info()[0:-1]
+                            print("error in concentrations")
+                            print(sys.exc_info()[0:-1])
                             #return
                 self._concentrationsAsAscii=self._toolConversion.getConcentrationsAsAscii(concentrations)
                 if len(self._concentrationsAsAscii) > 1:
@@ -540,25 +554,25 @@ class McaAdvancedFitBatch:
             #output options
             # .FIT files
             if self.fitFiles:
-                fitdir = os.path.join(self._outputdir,"FIT")
+                fitdir = self.os_path_join(self._outputdir,"FIT")
                 if not os.path.exists(fitdir):
                     try:
                         os.mkdir(fitdir)
                     except:
-                        print "I could not create directory %s" % fitdir
+                        print("I could not create directory %s" % fitdir)
                         return
-                fitdir = os.path.join(fitdir,filename+"_FITDIR")
+                fitdir = self.os_path_join(fitdir,filename+"_FITDIR")
                 if not os.path.exists(fitdir):
                     try:
                         os.mkdir(fitdir)
                     except:
-                        print "I could not create directory %s" % fitdir
+                        print("I could not create directory %s" % fitdir)
                         return
                 if not os.path.isdir(fitdir):
-                    print "%s does not seem to be a valid directory" % fitdir
+                    print("%s does not seem to be a valid directory" % fitdir)
                 else:
                     outfile = filename +"_"+key+".fit" 
-                    outfile = os.path.join(fitdir,  outfile)
+                    outfile = self.os_path_join(fitdir,  outfile)
                 if not useExistingResult:
                     result = self.mcafit.digestresult(outfile=outfile,
                                                       info=info)
@@ -570,16 +584,16 @@ class McaAdvancedFitBatch:
                         try:
                             os.remove(outfile)
                         except:
-                            print "error deleting fit file"
+                            print("error deleting fit file")
                         f.write(outfile)
                     except:
-                        print "Error writing concentrations to fit file"
-                        print sys.exc_info()
+                        print("Error writing concentrations to fit file")
+                        print(sys.exc_info())
 
                 #python like output list
                 if not self.counter:
                     name = os.path.splitext(self._rootname)[0]+"_fitfilelist.py"
-                    name = os.path.join(self._outputdir,name)
+                    name = self.os_path_join(self._outputdir,name)
                     try:
                         os.remove(name)
                     except:
@@ -602,15 +616,17 @@ class McaAdvancedFitBatch:
                 #this only works with EDF
                 if self.__ncols is not None:
                     if not self.counter:
-                        imgdir = os.path.join(self._outputdir,"IMAGES")
+                        imgdir = self.os_path_join(self._outputdir,"IMAGES")
                         if not os.path.exists(imgdir):
                             try:
                                 os.mkdir(imgdir)
                             except:
-                                print "I could not create directory %s" % imgdir
+                                print("I could not create directory %s" %\
+                                      imgdir)
                                 return
                         elif not os.path.isdir(imgdir):
-                            print "%s does not seem to be a valid directory" % imgdir
+                            print("%s does not seem to be a valid directory" %\
+                                  imgdir)
                         self.imgDir = imgdir
                         self.__peaks  = []
                         self.__images = {}
@@ -619,12 +635,18 @@ class McaAdvancedFitBatch:
                             self.__nrows   = len(range(0,len(self._filelist),self.fileStep))
                         for group in result['groups']:
                             self.__peaks.append(group)
-                            self.__images[group]=Numeric.zeros((self.__nrows,self.__ncols),Numeric.Float)
-                            self.__sigmas[group]=Numeric.zeros((self.__nrows,self.__ncols),Numeric.Float)
-                        self.__images['chisq']  = Numeric.zeros((self.__nrows,self.__ncols),Numeric.Float) - 1.
+                            self.__images[group]= numpy.zeros((self.__nrows,
+                                                               self.__ncols),
+                                                               numpy.float)
+                            self.__sigmas[group]= numpy.zeros((self.__nrows,
+                                                               self.__ncols),
+                                                               numpy.float)
+                        self.__images['chisq']  = numpy.zeros((self.__nrows,
+                                                               self.__ncols),
+                                                               numpy.float) - 1.
                         if self._concentrations:
                             layerlist = concentrations['layerlist']
-                            if concentrations.has_key('mmolar'):
+                            if 'mmolar' in concentrations:
                                 self.__conLabel = " mM"
                                 self.__conKey   = "mmolar"
                             else:
@@ -633,14 +655,16 @@ class McaAdvancedFitBatch:
                             for group in concentrations['groups']:
                                 key = group+self.__conLabel
                                 self.__concentrationsKeys.append(key)
-                                self.__images[key] = Numeric.zeros((self.__nrows,self.__ncols),
-                                                                                      Numeric.Float)
+                                self.__images[key] = numpy.zeros((self.__nrows,
+                                                                  self.__ncols),
+                                                                  numpy.float)
                                 if len(layerlist) > 1:
                                     for layer in layerlist:
                                         key = group+" "+layer
                                         self.__concentrationsKeys.append(key)                                        
-                                        self.__images[key] = Numeric.zeros((self.__nrows,self.__ncols),
-                                                                                      Numeric.Float)
+                                        self.__images[key] = numpy.zeros((self.__nrows,
+                                                                    self.__ncols),
+                                                                    numpy.float)
                 for peak in self.__peaks:
                     try:
                         self.__images[peak][self.__row, self.__col] = result[peak]['fitarea']
@@ -659,8 +683,9 @@ class McaAdvancedFitBatch:
                 try:
                     self.__images['chisq'][self.__row, self.__col] = result['chisq']
                 except:
-                    print "Error on chisq row %d col %d\n" % (self.__row, self.__col)
-                    print "File = %s\n", filename
+                    print("Error on chisq row %d col %d" %\
+                          (self.__row, self.__col))
+                    print("File = %s\n" % filename)
                     pass
 
         else:
@@ -668,15 +693,17 @@ class McaAdvancedFitBatch:
                 #this only works with EDF
                 if self.__ncols is not None:
                     if not self.counter:
-                        imgdir = os.path.join(self._outputdir,"IMAGES")
+                        imgdir = self.os_path_join(self._outputdir,"IMAGES")
                         if not os.path.exists(imgdir):
                             try:
                                 os.mkdir(imgdir)
                             except:
-                                print "I could not create directory %s" % imgdir
+                                print("I could not create directory %s" %\
+                                      imgdir)
                                 return
                         elif not os.path.isdir(imgdir):
-                            print "%s does not seem to be a valid directory"
+                            print("%s does not seem to be a valid directory" %\
+                                  imgdir)
                         self.imgDir = imgdir
                         self.__ROIpeaks  = []
                         self._ROIimages = {}
@@ -686,18 +713,20 @@ class McaAdvancedFitBatch:
                             self.__ROIpeaks.append(group)
                             self._ROIimages[group]={}
                             for roi in dict[group].keys():
-                                self._ROIimages[group][roi]=Numeric.zeros((self.__nrows,
-                                                                            self.__ncols),Numeric.Float)
+                                self._ROIimages[group][roi]=numpy.zeros((self.__nrows,
+                                                                   self.__ncols),
+                                                                   numpy.float)
                                 
                 if not hasattr(self, "_ROIimages"):
-                    print "ROI fitting only supported on EDF"
+                    print("ROI fitting only supported on EDF")
                 for group in self.__ROIpeaks:
                     for roi in self._ROIimages[group].keys():
                         try:
                             self._ROIimages[group][roi][self.__row, self.__col] = dict[group][roi]
                         except:
-                            print "error on (row,col) = ",self.__row, self.__col
-                            print "File = %s\n", filename
+                            print("error on (row,col) = %d,%d" %\
+                                  (self.__row, self.__col))
+                            print("File = %s" % filename)
                             pass
 
         #update counter
@@ -708,7 +737,7 @@ class McaAdvancedFitBatch:
         self.savedImages=[]
         if ffile is None:
             ffile = os.path.splitext(self._rootname)[0]
-            ffile = os.path.join(self.imgDir,ffile)
+            ffile = self.os_path_join(self.imgDir,ffile)
         if not self.roiFit:
             if (self.fileStep > 1) or (self.mcaStep > 1):
                 trailing = "_filestep_%02d_mcastep_%02d" % ( self.fileStep,
@@ -739,16 +768,25 @@ class McaAdvancedFitBatch:
                     speclabel +="  %s" % (peak)
                     edfname = ffile +"_"+peak+trailing+suffix
                 else:
-                    print "Unhandled peak name: %s. Not saved." % peak
+                    print("Unhandled peak name: %s. Not saved." % peak)
                     continue
                 dirname = os.path.dirname(edfname)
                 if not os.path.exists(dirname):
                     try:
                         os.mkdir(dirname)
                     except:
-                        print "I could not create directory % s" % dirname
-                edfout   = EdfFile.EdfFile(edfname)
-                edfout.WriteImage ({'Title':peak} , self.__images[peak], Append=0)
+                        print("I could not create directory %s" % dirname)
+                Append = 0
+                if os.path.exists(edfname):
+                    try:
+                        os.remove(edfname)
+                    except:
+                        print("I cannot delete output file")
+                        print("trying to append image to the end")
+                        Append = 1 
+                edfout   = EdfFile.EdfFile(edfname, access='ab')
+                edfout.WriteImage ({'Title':peak} , self.__images[peak], Append=Append)
+                edfout = None
                 self.savedImages.append(edfname)
             #save specfile format
             if self.chunk is None:
@@ -801,7 +839,7 @@ class McaAdvancedFitBatch:
                         try:
                             os.mkdir(dirname)
                         except:
-                            print "I could not create directory % s" % dirname
+                            print("I could not create directory %s" % dirname)
                     edfout  = EdfFile.EdfFile(edfname)
                     edfout.WriteImage ({'Title':group+" "+roi} , self._ROIimages[group][roi],
                                          Append=i)
@@ -834,7 +872,7 @@ if __name__ == "__main__":
             roiwidth = float(arg)
     filelist=args
     if len(filelist) == 0:
-        print "No input files, run GUI"
+        print("No input files, run GUI")
         sys.exit(0)
     
     b = McaAdvancedFitBatch(cfg,filelist,outdir,roifit,roiwidth)
