@@ -1,54 +1,65 @@
 #/*##########################################################################
-# Copyright (C) 2004-2010 European Synchrotron Radiation Facility
+# Copyright (C) 2004-2012 European Synchrotron Radiation Facility
 #
-# This file is part of the PyMCA X-ray Fluorescence Toolkit developed at
-# the ESRF by the Beamline Instrumentation Software Support (BLISS) group.
+# This file is part of the PyMca X-ray Fluorescence Toolkit developed at
+# the ESRF by the Software group.
 #
-# This toolkit is free software; you can redistribute it and/or modify it 
+# This toolkit is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by the Free
-# Software Foundation; either version 2 of the License, or (at your option) 
+# Software Foundation; either version 2 of the License, or (at your option)
 # any later version.
 #
-# PyMCA is distributed in the hope that it will be useful, but WITHOUT ANY
+# PyMca is distributed in the hope that it will be useful, but WITHOUT ANY
 # WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 # FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 # details.
 #
 # You should have received a copy of the GNU General Public License along with
-# PyMCA; if not, write to the Free Software Foundation, Inc., 59 Temple Place,
-# Suite 330, Boston, MA 02111-1307, USA.
+# PyMca; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #
-# PyMCA follows the dual licensing model of Trolltech's Qt and Riverbank's PyQt
-# and cannot be used as a free plugin for a non-free program. 
+# PyMca follows the dual licensing model of Riverbank's PyQt and cannot be
+# used as a free plugin for a non-free program.
 #
-# Please contact the ESRF industrial unit (industry@esrf.fr) if this license 
+# Please contact the ESRF industrial unit (industry@esrf.fr) if this license
 # is a problem for you.
 #############################################################################*/
-import os
 import sys
-import PyQt4.QtCore as QtCore
-import PyQt4.QtGui as QtGui
-import h5py
-import HDF5Widget
-import HDF5Info
-import HDF5CounterTable
-import HDF5DatasetTable
+import os
 import posixpath
 import weakref
 import gc
-import ConfigDict
+import h5py
+try:
+    from PyMca import PyMcaQt as qt
+    safe_str = qt.safe_str
+except ImportError:
+    # for people using this widget without PyMca installed
+    import PyQt4.Qt as qt
+    safe_str = str
+
+if hasattr(qt, 'QString'):
+    QString = qt.QString
+else:
+    QString = str
+
+from PyMca import HDF5Widget
+from PyMca import HDF5Info
+from PyMca import HDF5CounterTable
+from PyMca import HDF5DatasetTable
+from PyMca import ConfigDict
 if "PyMcaDirs" in sys.modules:
-    import PyMcaDirs
+    from PyMca import PyMcaDirs
 
 DEBUG=0
 
-class Buttons(QtGui.QWidget):
+class Buttons(qt.QWidget):
     def __init__(self, parent=None, options=None):
-        QtGui.QWidget.__init__(self, parent)
-        self.mainLayout = QtGui.QGridLayout(self)
+        qt.QWidget.__init__(self, parent)
+        self.mainLayout = qt.QGridLayout(self)
         self.mainLayout.setMargin(0)
         self.mainLayout.setSpacing(2)
-        self.buttonGroup = QtGui.QButtonGroup(self)
+        self.buttonGroup = qt.QButtonGroup(self)
         self.buttonList = []
         i = 0
         if options is None:
@@ -60,24 +71,24 @@ class Buttons(QtGui.QWidget):
             row = optionList.index(option)
             for action in actionList:
                 col = actionList.index(action)
-                button = QtGui.QPushButton(self)
+                button = qt.QPushButton(self)
                 button.setText(action + " " + option)
                 self.mainLayout.addWidget(button, row, col)
                 self.buttonGroup.addButton(button)
                 self.buttonList.append(button)
         self.connect(self.buttonGroup,
-                     QtCore.SIGNAL('buttonClicked(QAbstractButton *)'),
+                     qt.SIGNAL('buttonClicked(QAbstractButton *)'),
                      self.emitSignal)
 
     def emitSignal(self, button):
         ddict={}
         ddict['event'] = 'buttonClicked'
-        ddict['action'] = str(button.text())
-        self.emit(QtCore.SIGNAL('ButtonsSignal'), ddict)
+        ddict['action'] = safe_str(button.text())
+        self.emit(qt.SIGNAL('ButtonsSignal'), ddict)
 
-class QNexusWidget(QtGui.QWidget):
+class QNexusWidget(qt.QWidget):
     def __init__(self, parent=None):
-        QtGui.QWidget.__init__(self, parent)
+        qt.QWidget.__init__(self, parent)
         self.data = None
         self._dataSourceList = []
         self._oldCntSelection = None
@@ -93,16 +104,17 @@ class QNexusWidget(QtGui.QWidget):
         self.build()
 
     def build(self):
-        self.mainLayout = QtGui.QVBoxLayout(self)
-        self.splitter = QtGui.QSplitter(self)
-        self.splitter.setOrientation(QtCore.Qt.Vertical)
+        self.mainLayout = qt.QVBoxLayout(self)
+        self.splitter = qt.QSplitter(self)
+        self.splitter.setOrientation(qt.Qt.Vertical)
         self.hdf5Widget = HDF5Widget.HDF5Widget(self._defaultModel,
                                                 self.splitter)
-        self.hdf5Widget.setSelectionMode(QtGui.QAbstractItemView.ExtendedSelection)
+        self.hdf5Widget.setSelectionMode(qt.QAbstractItemView.ExtendedSelection)
         self.cntTable = HDF5CounterTable.HDF5CounterTable(self.splitter)
         self.mainLayout.addWidget(self.splitter)
         #Enable 3D
-        if 'Object3D' in sys.modules:
+        if ('PyMca.Object3D' in sys.modules) or \
+           ('Object3D' in sys.modules):
             self.buttons = Buttons(self, options=['SCAN', 'MCA', '2D', '3D'])
             self.cntTable.set3DEnabled(True)
         else:
@@ -110,42 +122,37 @@ class QNexusWidget(QtGui.QWidget):
             self.cntTable.set3DEnabled(False)
         self.mainLayout.addWidget(self.buttons)
         self.connect(self.hdf5Widget,
-                     QtCore.SIGNAL('HDF5WidgetSignal'),
+                     qt.SIGNAL('HDF5WidgetSignal'),
                      self.hdf5Slot)
         self.connect(self.cntTable,
-                     QtCore.SIGNAL('customContextMenuRequested(QPoint)'),
+                     qt.SIGNAL('customContextMenuRequested(QPoint)'),
                      self._counterTableCustomMenuSlot)
         self.connect(self.buttons,
-                     QtCore.SIGNAL('ButtonsSignal'),
+                     qt.SIGNAL('ButtonsSignal'),
                      self.buttonsSlot)
 
-        self._hdf5WidgetDatasetMenu = QtGui.QMenu(self)
-        self._hdf5WidgetDatasetMenu.addAction(QtCore.QString("Open"),
-                                    self._openDataset)
-        self._hdf5WidgetDatasetMenu.addAction(QtCore.QString("Show Properties"),
-                                    self._showDatasetProperties)
 
         # Some convenience functions to customize the table
         # They could have been included in other class inheriting
         # this one.
-        self.cntTable.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
+        self.cntTable.setContextMenuPolicy(qt.Qt.CustomContextMenu)
 
-        self._cntTableMenu = QtGui.QMenu(self)
-        self._cntTableMenu.addAction(QtCore.QString("Load"),
+        self._cntTableMenu = qt.QMenu(self)
+        self._cntTableMenu.addAction(QString("Load"),
                                     self._loadCounterTableConfiguration)
-        self._cntTableMenu.addAction(QtCore.QString("Merge"),
+        self._cntTableMenu.addAction(QString("Merge"),
                                     self._mergeCounterTableConfiguration)
-        self._cntTableMenu.addAction(QtCore.QString("Save"),
+        self._cntTableMenu.addAction(QString("Save"),
                                     self._saveCounterTableConfiguration)
         self._cntTableMenu.addSeparator()
-        self._cntTableMenu.addAction(QtCore.QString("Delete All"),
+        self._cntTableMenu.addAction(QString("Delete All"),
                                     self._deleteAllCountersFromTable)
-        self._cntTableMenu.addAction(QtCore.QString("Delete Selected"),
+        self._cntTableMenu.addAction(QString("Delete Selected"),
                                     self._deleteSelectedCountersFromTable)
 
     def _counterTableCustomMenuSlot(self, qpoint):
         self.getWidgetConfiguration()
-        self._cntTableMenu.exec_(QtGui.QCursor.pos())
+        self._cntTableMenu.exec_(qt.QCursor.pos())
 
     def _getConfigurationFromFile(self, fname):
         ddict = ConfigDict.ConfigDict()
@@ -156,15 +163,15 @@ class QNexusWidget(QtGui.QWidget):
             ddict = ddict['PyMca']
         
         if 'HDF5' not in ddict.keys():
-            msg = QtGui.QMessageBox(self)
-            msg.setIcon(QtGui.QMessageBox.Information)
+            msg = qt.QMessageBox(self)
+            msg.setIcon(qt.QMessageBox.Information)
             msg.setText("File does not contain HDF5 configuration")
             msg.exec_()
             return None
 
         if 'WidgetConfiguration' not in ddict['HDF5'].keys():
-            msg = QtGui.QMessageBox(self)
-            msg.setIcon(QtGui.QMessageBox.Information)
+            msg = qt.QMessageBox(self)
+            msg.setIcon(qt.QMessageBox.Information)
             msg.setText("File does not contain HDF5 WidgetConfiguration")
             msg.exec_()
             return None
@@ -174,15 +181,15 @@ class QNexusWidget(QtGui.QWidget):
 
         if ('counters' not in keys) or\
            ('aliases' not in keys):
-            msg = QtGui.QMessageBox(self)
-            msg.setIcon(QtGui.QMessageBox.Information)
+            msg = qt.QMessageBox(self)
+            msg.setIcon(qt.QMessageBox.Information)
             msg.setText("File does not contain HDF5 counters information")
             msg.exec_()
             return None
 
         if len(ddict['counters']) != len(ddict['aliases']):
-            msg = QtGui.QMessageBox(self)
-            msg.setIcon(QtGui.QMessageBox.Critical)
+            msg = qt.QMessageBox(self)
+            msg.setIcon(qt.QMessageBox.Critical)
             msg.setText("Number of counters does not match number of aliases")
             msg.exec_()
             return None
@@ -266,8 +273,8 @@ class QNexusWidget(QtGui.QWidget):
         ddict['aliases'] = []
         for i in range(self.cntTable.rowCount()):
             if i not in rowList:
-                name = str(self.cntTable.item(i, 0).text())
-                alias = str(self.cntTable.item(i, 4).text())
+                name = safe_str(self.cntTable.item(i, 0).text())
+                alias = safe_str(self.cntTable.item(i, 4).text())
                 ddict['counters'].append(name)
                 ddict['aliases'].append(alias)
 
@@ -286,7 +293,7 @@ class QNexusWidget(QtGui.QWidget):
         if not os.path.exists(inidir):
             inidir = os.getcwd()
 
-        ret = str(QtGui.QFileDialog.getOpenFileName(self,
+        ret = safe_str(qt.QFileDialog.getOpenFileName(self,
                                          "Select a .ini file",
                                          inidir,
                                          "*.ini"))
@@ -308,7 +315,7 @@ class QNexusWidget(QtGui.QWidget):
         if not os.path.exists(inidir):
             inidir = os.getcwd()
 
-        ret = str(QtGui.QFileDialog.getSaveFileName(self,
+        ret = safe_str(qt.QFileDialog.getSaveFileName(self,
                                          "Select a .ini file",
                                          inidir,
                                          "*.ini"))
@@ -344,11 +351,12 @@ class QNexusWidget(QtGui.QWidget):
             self.hdf5Widget.collapseAll()
             self.hdf5Widget.setModel(self._defaultModel)
             return
-        def dataSourceDistroyed(weakrefReference):
+        def dataSourceDestroyed(weakrefReference):
             idx = self._dataSourceList.index(weakrefReference)
             del self._dataSourceList[idx]
             del self._modelDict[weakrefReference]
-        ref = weakref.ref(dataSource, dataSourceDistroyed)
+            return
+        ref = weakref.ref(dataSource, dataSourceDestroyed)
         if ref not in self._dataSourceList:
             self._dataSourceList.append(ref)
             self._modelDict[ref] = HDF5Widget.FileModel()
@@ -362,8 +370,19 @@ class QNexusWidget(QtGui.QWidget):
 
     def showInfoWidget(self, filename, name, dset=False):
         self._checkWidgetDict()
-        fileIndex = self.data.sourceName.index(filename)
-        phynxFile  = self.data._sourceObjectList[fileIndex]
+        #this solution seems more robust
+        if 1:
+            useInstance = True
+        else:
+            if h5py.version.version < '2.0':
+                useInstance = True
+            else:
+                useInstance = False
+        if useInstance:
+            fileIndex = self.data.sourceName.index(filename)
+            phynxFile  = self.data._sourceObjectList[fileIndex]
+        else:
+            phynxFile  = h5py.File(filename, 'r')
         info = self.getInfo(phynxFile, name)
         widget = HDF5Info.HDF5InfoWidget()
         widget.notifyCloseEventToWidget(self)
@@ -380,41 +399,80 @@ class QNexusWidget(QtGui.QWidget):
                 pass
         self._lastWidgetId = wid
         self._widgetDict[wid] = widget
+        if useInstance:
+            def sourceObjectDestroyed(weakrefReference):
+                if wid == self._lastWidgetId:
+                    self._latWidgetId = None
+                if wid in self._widgetDict:
+                    del self._widgetDict[wid]
+            widget._sourceObjectWeakReference = weakref.ref(phynxFile,
+                                                 sourceObjectDestroyed)
         widget.setInfoDict(info)
         if dset:
             dataset = phynxFile[name]
             if isinstance(dataset, h5py.Dataset):
-                widget.w = HDF5DatasetTable.HDF5DatasetTable(widget)
-                try:
-                    widget.w.setDataset(dataset)
-                except:
-                    print "Error filling table"
-                widget.addTab(widget.w, 'TableView')
+                if len(dataset.shape):
+                    #0 length datasets do not need a table
+                    widget.w = HDF5DatasetTable.HDF5DatasetTable(widget)
+                    try:
+                        widget.w.setDataset(dataset)
+                    except:
+                        print("Error filling table")
+                    widget.addTab(widget.w, 'TableView')
         widget.show()
         return widget
 
     def itemRightClickedSlot(self, ddict):
         filename = ddict['file']
         name = ddict['name']
-        if True:
-            if ddict['dtype'].startswith('|S'):
-                #print "string"
-                return self.showInfoWidget(filename, name, False)
-                pass
-            elif 1:
-                #should I show the option menu?
-                self.showInfoWidget(filename, name, True)
-                return
+        if ddict['dtype'].startswith('|S'):
+            #handle a right click on a dataset of string type
+            return self.showInfoWidget(filename, name, False)
+            pass
+        elif ddict['dtype'] == '':
+            #handle a right click on a group
+            return self.showInfoWidget(filename, name, False)
+        elif 0:
+            #should I show the option menu?
+            self.showInfoWidget(filename, name, True)
+            return
+        else:
+            #handle a right click on a numeric dataset
+            _hdf5WidgetDatasetMenu = qt.QMenu(self)
+            _hdf5WidgetDatasetMenu.addAction(QString("Add to selection table"),
+                                        self._addToSelectionTable)
+
+            if 0:
+                #these two options can be combined into one for the time being
+                _hdf5WidgetDatasetMenu.addAction(QString("Open"),
+                                            self._openDataset)
+                _hdf5WidgetDatasetMenu.addAction(QString("Show Properties"),
+                                            self._showDatasetProperties)
             else:
-                self.__lastDatasetDict= ddict
-                self._hdf5WidgetDatasetMenu.exec_(QtGui.QCursor.pos())
-                self.__lastDatasetDict= None
-                return
-        return self.showInfoWidget(filename, name)
+                _hdf5WidgetDatasetMenu.addAction(QString("Show Information"),
+                                        self._showInfoWidgetSlot)
+                self._lastDatasetDict= ddict
+                _hdf5WidgetDatasetMenu.exec_(qt.QCursor.pos())
+                self._lastDatasetDict= None
+            return
+
+    def _addToSelectionTable(self, ddict=None):
+        if ddict is None:
+            ddict = self._lastDatasetDict
+        #handle as a double click
+        ddict['event'] = "itemDoubleClicked"
+        self.hdf5Slot(ddict)
+
+    def _showInfoWidgetSlot(self, ddict=None):
+        if ddict is None:
+            ddict = self._lastDatasetDict
+        filename = ddict['file']
+        name = ddict['name']
+        return self.showInfoWidget(filename, name, True)
 
     def _openDataset(self, ddict=None):
         if ddict is None:
-            ddict = self.__lastDatasetDict
+            ddict = self._lastDatasetDict
         filename = ddict['file']
         name = ddict['name']
         self._checkWidgetDict()
@@ -451,7 +509,7 @@ class QNexusWidget(QtGui.QWidget):
 
     def _showDatasetProperties(self, ddict=None):
         if ddict is None:
-            ddict = self.__lastDatasetDict
+            ddict = self._lastDatasetDict
         filename = ddict['file']
         name = ddict['name']
         return self.showInfoWidget(filename, name)
@@ -463,7 +521,7 @@ class QNexusWidget(QtGui.QWidget):
         if ddict['event'] == "itemDoubleClicked":
             if ddict['type'] in ['Dataset']:
                 if ddict['dtype'].startswith('|S'):
-                    print "string"
+                    print("string")
                 else:
                     root = ddict['name'].split('/')
                     root = "/" + root[1]
@@ -479,7 +537,7 @@ class QNexusWidget(QtGui.QWidget):
                         else:
                             self._aliasList.append(cnt)
                         self.cntTable.build(self._cntList, self._aliasList)
-            elif ddict['type'] == 'Entry':
+            elif ddict['type'] in ['NXentry', 'Entry']:
                 if self._lastAction is None:
                     return
                 action, selectionType = self._lastAction.split()
@@ -500,7 +558,7 @@ class QNexusWidget(QtGui.QWidget):
                         cnt  = ddict['name'].split(root)[-1]
                         if cnt not in self._cntList:
                             if DEBUG:
-                                print "USING SECOND WAY"
+                                print("USING SECOND WAY")
                             self._cntList.append(cnt)
                             basename = posixpath.basename(cnt)
                             if basename not in self._aliasList:
@@ -510,13 +568,13 @@ class QNexusWidget(QtGui.QWidget):
                             self.cntTable.build(self._cntList, self._aliasList)
                         return
                 if DEBUG:
-                    print "Unhandled item type:", ddict['dtype']
+                    print("Unhandled item type: %s" % ddict['dtype'])
 
     def buttonsSlot(self, ddict):
         if self.data is None:
             return
         action, selectionType = ddict['action'].split()
-        entryList = self.getEntryList()
+        entryList = self.getSelectedEntries()
         if not len(entryList):
             return
         cntSelection = self.cntTable.getCounterSelection()
@@ -534,10 +592,10 @@ class QNexusWidget(QtGui.QWidget):
                 sel['SourceType'] = "HDF5"
                 fileIndex = self.data.sourceName.index(filename)
                 phynxFile  = self.data._sourceObjectList[fileIndex]
-                entryIndex = phynxFile["/"].keys().index(entry[1:])
+                entryIndex = list(phynxFile["/"].keys()).index(entry[1:])
                 sel['Key']        = "%d.%d" % (fileIndex+1, entryIndex+1)
                 sel['legend']     = os.path.basename(sel['SourceName'][0])+\
-                                    " " + sel['Key']
+                                    " " + posixpath.basename(entry) #it was sel['Key']
                 sel['selection'] = {}
                 sel['selection']['sourcename'] = filename
                 #deal with the case the "entry" is a dataset hunging at root level
@@ -582,15 +640,15 @@ class QNexusWidget(QtGui.QWidget):
                 ddict = {}
                 ddict['event'] = "SelectionTypeChanged"
                 ddict['SelectionType'] = selectionType.upper()
-                self.emit(QtCore.SIGNAL('otherSignals'), ddict)
+                self.emit(qt.SIGNAL('otherSignals'), ddict)
             if action.upper() == "ADD":
-                self.emit(QtCore.SIGNAL("addSelection"), selectionList)
+                self.emit(qt.SIGNAL("addSelection"), selectionList)
             if action.upper() == "REMOVE":
-                self.emit(QtCore.SIGNAL("removeSelection"), selectionList)
+                self.emit(qt.SIGNAL("removeSelection"), selectionList)
             if action.upper() == "REPLACE":
-                self.emit(QtCore.SIGNAL("replaceSelection"), selectionList)
+                self.emit(qt.SIGNAL("replaceSelection"), selectionList)
 
-    def getEntryList(self):
+    def getSelectedEntries(self):
         return self.hdf5Widget.getSelectedEntries()
 
     def closeEvent(self, event):
@@ -598,7 +656,7 @@ class QNexusWidget(QtGui.QWidget):
         for key in keyList:
             self._widgetDict[key].close()
             del self._widgetDict[key]
-        return QtGui.QWidget.closeEvent(self, event)
+        return qt.QWidget.closeEvent(self, event)
 
     def _checkWidgetDict(self):
         keyList = self._widgetDict.keys()
@@ -609,33 +667,40 @@ class QNexusWidget(QtGui.QWidget):
     def customEvent(self, event):
         if hasattr(event, 'dict'):
             ddict = event.dict
-            if ddict.has_key('event'):
+            if 'event' in ddict:
                 if ddict['event'] == "closeEventSignal":
-                    if self._widgetDict.has_key(ddict['id']):
+                    if ddict['id'] in self._widgetDict:
+                        if DEBUG:
+                            try:
+                                widget = self._widgetDict[ddict['id']] 
+                                print("DELETING %s" % widget.windowTitle())
+                            except:
+                                pass
                         del self._widgetDict[ddict['id']]
     
 if __name__ == "__main__":
     import sys
-    app = QtGui.QApplication(sys.argv)
+    app = qt.QApplication(sys.argv)
     try:
-        import Object3D
+        #this is to add the 3D buttons ...
+        from PyMca import Object3D
     except:
         pass
     w = QNexusWidget()
     if 0:
         w.setFile(sys.argv[1])
     else:
-        import NexusDataSource
-        dataSource = NexusDataSource.NexusDataSource(sys.argv[1])
+        from PyMca import NexusDataSource
+        dataSource = NexusDataSource.NexusDataSource(sys.argv[1:])
         w.setDataSource(dataSource)
     def addSelection(sel):
-        print sel
+        print(sel)
     def removeSelection(sel):
-        print sel
+        print(sel)
     def replaceSelection(sel):
-        print sel
+        print(sel)
     w.show()
-    QtCore.QObject.connect(w, QtCore.SIGNAL("addSelection"),     addSelection)
-    QtCore.QObject.connect(w, QtCore.SIGNAL("removeSelection"),  removeSelection)
-    QtCore.QObject.connect(w, QtCore.SIGNAL("replaceSelection"), replaceSelection)
+    qt.QObject.connect(w, qt.SIGNAL("addSelection"),     addSelection)
+    qt.QObject.connect(w, qt.SIGNAL("removeSelection"),  removeSelection)
+    qt.QObject.connect(w, qt.SIGNAL("replaceSelection"), replaceSelection)
     sys.exit(app.exec_())
