@@ -1,59 +1,49 @@
 #!/usr/bin/env python
 #/*##########################################################################
-# Copyright (C) 2004-2010 European Synchrotron Radiation Facility
+# Copyright (C) 2004-2012 European Synchrotron Radiation Facility
 #
-# This file is part of the PyMCA X-ray Fluorescence Toolkit developed at
-# the ESRF by the Beamline Instrumentation Software Support (BLISS) group.
+# This file is part of the PyMca X-ray Fluorescence Toolkit developed at
+# the ESRF by the Software group.
 #
-# This toolkit is free software; you can redistribute it and/or modify it 
+# This toolkit is free software; you can redistribute it and/or modify it
 # under the terms of the GNU General Public License as published by the Free
-# Software Foundation; either version 2 of the License, or (at your option) 
+# Software Foundation; either version 2 of the License, or (at your option)
 # any later version.
 #
-# PyMCA is distributed in the hope that it will be useful, but WITHOUT ANY
+# PyMca is distributed in the hope that it will be useful, but WITHOUT ANY
 # WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 # FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 # details.
 #
 # You should have received a copy of the GNU General Public License along with
-# PyMCA; if not, write to the Free Software Foundation, Inc., 59 Temple Place,
-# Suite 330, Boston, MA 02111-1307, USA.
+# PyMca; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 #
-# PyMCA follows the dual licensing model of Trolltech's Qt and Riverbank's PyQt
-# and cannot be used as a free plugin for a non-free program. 
+# PyMca follows the dual licensing model of Riverbank's PyQt and cannot be
+# used as a free plugin for a non-free program.
 #
-# Please contact the ESRF industrial unit (industry@esrf.fr) if this license 
+# Please contact the ESRF industrial unit (industry@esrf.fr) if this license
 # is a problem for you.
 #############################################################################*/
 __author__ = "V.A. Sole - ESRF Software Group"
-import PyMcaQt as qt
 import sys
 import os
 import numpy
+
 from matplotlib import cm
 from matplotlib.font_manager import FontProperties
 from matplotlib.backends.backend_qt4agg import FigureCanvasQTAgg as FigureCanvas
 from matplotlib.figure import Figure
 from matplotlib.colors import LinearSegmentedColormap, LogNorm, Normalize
-import PyMcaMatplotlibSave
-from PyMca_Icons import IconDict
-import PyMcaPrintPreview
-import PyMcaDirs
+
+from PyMca import PyMcaQt as qt
+from PyMca import PyMcaMatplotlibSave
+from PyMca.PyMca_Icons import IconDict
+from PyMca import PyMcaPrintPreview
+from PyMca import PyMcaDirs
 
 DEBUG = 0
 
-class HorizontalSpacer(qt.QWidget):
-    def __init__(self, *args):
-        qt.QWidget.__init__(self, *args)
-
-        self.setSizePolicy(qt.QSizePolicy(qt.QSizePolicy.Expanding,
-                                          qt.QSizePolicy.Fixed))
-
-class VerticalSpacer(qt.QWidget):
-    def __init__(self, *args):
-        qt.QWidget.__init__(self, *args)
-        self.setSizePolicy(qt.QSizePolicy(qt.QSizePolicy.Fixed,
-                                          qt.QSizePolicy.Expanding))
 
 class TopWidget(qt.QWidget):
     def __init__(self, parent = None):
@@ -76,7 +66,7 @@ class TopWidget(qt.QWidget):
         ddict = {}
         i = 0
         for label in self.keyList:
-            ddict[label] = str(self.lineEditList[i].text())
+            ddict[label] = qt.safe_str(self.lineEditList[i].text())
             i = i + 1
         return ddict
 
@@ -209,7 +199,10 @@ class SaveImageSetup(qt.QWidget):
             self.lastOutputDir = PyMcaDirs.outputDir
 
         outfile.setWindowTitle("Output File Selection")
-        strlist = qt.QStringList()
+        if hasattr(qt, "QStringList"):
+            strlist = qt.QStringList()
+        else:
+            strlist = []
         format_list = []
         format_list.append('Graphics PNG *.png')
         format_list.append('Graphics EPS *.eps')
@@ -223,11 +216,18 @@ class SaveImageSetup(qt.QWidget):
         outfile.setDirectory(self.lastOutputDir)
         ret = outfile.exec_()
         if ret:
-            filterused = str(outfile.selectedFilter()).split()
+            filterused = qt.safe_str(outfile.selectedFilter()).split()
             filedescription = filterused[0]
             filetype  = filterused[1]
             extension = filterused[2]
-            outstr=str(outfile.selectedFiles()[0])
+            try:
+                outstr = qt.safe_str(outfile.selectedFiles()[0])
+            except:
+                msg = qt.QMessageBox(self)
+                msg.setIcon(qt.QMessageBox.Critical)
+                msg.setText("Error saving image: %s" % sys.exc_info()[1])
+                msg.setWindowTitle('Matplotlib Save Image')
+                msg.exec_()
             try:            
                 outputDir  = os.path.dirname(outstr)
                 self.lastOutputDir = outputDir
@@ -265,9 +265,10 @@ class SaveImageSetup(qt.QWidget):
             self.imageWidget.print_figure(finalFile,
                                           edgecolor='w',
                                           facecolor='w',
-                                          format=finalFile[-3:])
+                                          format=finalFile[-3:],
+                                          dpi=self.imageWidget.config['outputdpi'])
         except:
-            print "WARNING: trying to save using obsolete method"
+            print("WARNING: trying to save using obsolete method")
             config = self.imageWidget.getParameters()
             try:
                 s=PyMcaMatplotlibSave.PyMcaMatplotlibSaveImage(self.imageWidget.imageData)
@@ -296,7 +297,7 @@ class SimpleComboBox(qt.QComboBox):
 
     def setCurrentText(self, text):
         for i in range(self.count()):
-            if str(self.itemText(i)) == text:
+            if qt.safe_str(self.itemText(i)) == text:
                 self.setCurrentIndex(i)
                 break
 
@@ -329,7 +330,8 @@ class RightWidget(qt.QWidget):
                         'Zoom Y Min',
                         'Zoom Y Max',
                         'Value Min',
-                        'Value Max']
+                        'Value Max',
+                        'Output dpi']
         self.keyList = []
         for label in self.labelList:
             self.keyList.append(label.lower().replace(' ','').replace('/',""))
@@ -387,15 +389,19 @@ class RightWidget(qt.QWidget):
                     tip += "limits to the same value."
                     line.setToolTip(tip)
                     line.setText('0.0')
+                elif 'Output dpi' in self.labelList[i]:
+                    tip  = "=Output file resolution."
+                    line.setToolTip(tip)
+                    line.setText("%d" % 100)
                 else:
                     line.setText('1.0')
             self.gridLayout.addWidget(label, i, 0)
             self.gridLayout.addWidget(line, i, 1)
             self.comboBoxList.append(line)
 
-	self.mainLayout.addWidget(self.gridWidget)
-	self.mainLayout.addWidget(VerticalSpacer(self))
-	self.setPixmapMode(False)
+        self.mainLayout.addWidget(self.gridWidget)
+        self.mainLayout.addWidget(qt.VerticalSpacer(self))
+        self.setPixmapMode(False)
 
     def setPixmapMode(self, flag):
         if flag:
@@ -413,21 +419,24 @@ class RightWidget(qt.QWidget):
                 self.comboBoxList[index].setEnabled(True)
 
     def getParameters(self):
-	ddict = {}
-	i = 0
-	for label in self.keyList:
+        ddict = {}
+        i = 0
+        for label in self.keyList:
             if i > self.labelList.index('Image Background'):
-                text = str(self.comboBoxList[i].text())
+                text = qt.safe_str(self.comboBoxList[i].text())
                 if len(text):
-                    ddict[label] = float(text)
+                    if label == 'Output dpi':
+                        ddict[label] = int(text)
+                    else:
+                        ddict[label] = float(text)
                 else:
                     ddict[label] = None
             else:
-                ddict[label] = str(self.comboBoxList[i].currentText()).lower()
-	    if (ddict[label] == 'none') or (ddict[label] == 'default'):
-		ddict[label] = None
-	    i = i + 1
-	return ddict
+                ddict[label] = qt.safe_str(self.comboBoxList[i].currentText()).lower()
+            if (ddict[label] == 'none') or (ddict[label] == 'default'):
+                ddict[label] = None
+            i = i + 1
+        return ddict
 
     def setParameters(self, ddict):
         for label in ddict.keys():
@@ -435,9 +444,19 @@ class RightWidget(qt.QWidget):
                 i = self.keyList.index(label)
                 if i > self.labelList.index('Image Background'):
                     if ddict[label] is not None:
-                        self.comboBoxList[i].setText("%f" % ddict[label])
+                        if label == 'Output dpi':
+                            self.comboBoxList[i].setText("%d" % int(ddict[label]))
+                        else:
+                            self.comboBoxList[i].setText("%f" % ddict[label])
                 else:
-                    self.comboBoxList[i].setCurrentText(ddict[label])
+                    txt = ddict[label]
+                    if ddict[label] is not None:
+                        try:
+                            txt = ddict[label][0].upper() +\
+                                  ddict[label][1:].lower()
+                        except:
+                            pass
+                    self.comboBoxList[i].setCurrentText(txt)
         return
 	
 class MyLineEdit(qt.QLineEdit):
@@ -513,7 +532,8 @@ class QPyMcaMatplotlibImage(FigureCanvas):
                      'valuemax':None,
                      'xlimits':xlimits,
                      'ylimits':ylimits,
-                     'vlimits':vlimits}
+                     'vlimits':vlimits,
+                     'outputdpi':dpi}
 
         #generate own colormaps
         cdict = {'red': ((0.0, 0.0, 0.0),
