@@ -1,5 +1,5 @@
 #/*##########################################################################
-# Copyright (C) 2004-2007 European Synchrotron Radiation Facility
+# Copyright (C) 2004-2012 European Synchrotron Radiation Facility
 #
 # This file is part of the PyMCA X-ray Fluorescence Toolkit developed at
 # the ESRF by the Beamline Instrumentation Software Support (BLISS) group.
@@ -22,7 +22,7 @@
 # and cannot be used as a free plugin for a non-free program. 
 #
 # Please contact the ESRF industrial unit (industry@esrf.fr) if this license 
-# is a problem to you.
+# is a problem for you.
 #############################################################################*/
 /*char RcsId[] = "$Header: /segfs/bliss/source/python/specfile/specfile-3.1/src/RCS/sfdata.c,v 1.8 2005/07/04 15:02:38 ahoms Exp $"; */
 /************************************************************************
@@ -74,6 +74,13 @@
  */
 #include <SpecFile.h>
 #include <SpecFileP.h>
+#include <locale_management.h>
+
+#ifndef _GNU_SOURCE
+#ifdef PYMCA_POSIX
+#include <locale.h>
+#endif
+#endif
 
 #include <ctype.h>
 /*
@@ -200,7 +207,13 @@ SfData( SpecFile *sf, long index, double ***retdata, long **retinfo, int *error
              maxcol=200;
      long    rows;
      int     i;
-     
+#ifndef _GNU_SOURCE
+#ifdef PYMCA_POSIX
+	char *currentLocaleBuffer;
+	char localeBuffer[21];
+#endif
+#endif
+
      if (index <= 0 ){
         return(-1);
      }
@@ -291,6 +304,13 @@ if(0){
      */
     ptr++;
 }
+#ifndef _GNU_SOURCE
+#ifdef PYMCA_POSIX
+	currentLocaleBuffer = setlocale(LC_NUMERIC, NULL);
+	strcpy(localeBuffer, currentLocaleBuffer);
+	setlocale(LC_NUMERIC, "C\0");
+#endif
+#endif
 
     for ( ; ptr < to; ptr++) {
         /* get a complete line */
@@ -344,7 +364,7 @@ if(0){
             if (*ptr == ' ' || *ptr == '\t' ) {
                 strval[i] = '\0';
                 i = 0;
-                val = atof(strval);
+                val = PyMcaAtof(strval);
                 valline[cols] = val;
                 cols++;
                 if (cols >= maxcol) return(-1);
@@ -362,7 +382,7 @@ if(0){
         }
         if ((*(ptr)== '\n') && (i != 0)){
                 strval[i] = '\0';
-                val = atof(strval);
+                val = PyMcaAtof(strval);
                 valline[cols] = val;
                 cols++;
                 if (cols >= maxcol) return(-1);
@@ -375,7 +395,8 @@ if(0){
         rows++;
         /*cols++;*/
         if (cols >= maxcol) return(-1);            
-       /* printf("Adding a new row, nrows = %ld, ncols= %ld\n",rows,cols);*/
+        /* printf("Adding a new row, nrows = %ld, ncols= %ld\n",rows,cols);*/
+        /*printf("info col = %d cols = %d\n", dinfo[COL], cols);*/ 
         if (dinfo[COL] != 0 && cols != dinfo[COL]) {
                     ;
                     /*diffract31 crash -> nextline uncommented */
@@ -389,12 +410,20 @@ if(0){
               data = (double **) realloc ( data, sizeof(double) * (rows+1));
               data[rows] = dataline;
               dinfo[ROW]=rows+1;
+        }else{
+              printf("Error on scan %d line %d\n", (int) index, (int) (rows+1));
+              /* just ignore the line instead of stopping there with a
+              break; */
+              rows--;
         }
         }
     }
 
-
-
+#ifndef _GNU_SOURCE
+#ifdef PYMCA_POSIX
+    setlocale(LC_NUMERIC, localeBuffer);
+#endif
+#endif
     /* 
     * make a copy in specfile structure
     */
